/*
 * This file contains low level routines for the HANDYSCOPE
 */
#include "HS.H"

word wADaddress;

word wMeasurementArray[NRCHANNELS][NRSAMPLES];
float rInputValue[NRSENSITIVITIES] = { 0.5, 1, 2, 5, 10, 20 };




void SetSensitivity( byte byInputRangeCh1 , byte byInputRangeCh2 )
/* DESCRIPTION
 *   This routine sets the sensitivity for both channels of the HANDYSCOPE
 * INPUT
 *   byte byInputRangeCh1   pointer to the sensitivity array element for Ch1
 *   byte byInputRangeCh2   pointer to the sensitivity array element for Ch2
 * OUTPUT
 *   -
 */
{
  byte bySettingCh1;
  byte bySettingCh2;
 /*
  * Convert sesitivity settings to bit patterns
  */
  switch (byInputRangeCh1)
  {
    case INPUTRANGE_05 : bySettingCh1 = 0xf8 + 0x0;  // 1111 1000
                         break;
    case INPUTRANGE_1  : bySettingCh1 = 0xf8 + 0x4;  // 1111 1100
                         break;
    case INPUTRANGE_2  : bySettingCh1 = 0xf8 + 0x6;  // 1111 1110
                         break;
    case INPUTRANGE_5  : bySettingCh1 = 0xf8 + 0x1;  // 1111 1001
                         break;
    case INPUTRANGE_10 : bySettingCh1 = 0xf8 + 0x5;  // 1111 1101
                         break;
    case INPUTRANGE_20 : bySettingCh1 = 0xf8 + 0x7;  // 1111 1111
                         break;
  } // switch
  switch (byInputRangeCh2)
  {
    case INPUTRANGE_05 : bySettingCh2 = 0xf8 + 0x0;  // 1111 1000
                         break;
    case INPUTRANGE_1  : bySettingCh2 = 0xf8 + 0x4;  // 1111 1100
                         break;
    case INPUTRANGE_2  : bySettingCh2 = 0xf8 + 0x6;  // 1111 1110
                         break;
    case INPUTRANGE_5  : bySettingCh2 = 0xf8 + 0x1;  // 1111 1001
                         break;
    case INPUTRANGE_10 : bySettingCh2 = 0xf8 + 0x5;  // 1111 1101
                         break;
    case INPUTRANGE_20 : bySettingCh2 = 0xf8 + 0x7;  // 1111 1111
                         break;
  } // switch
 /*
  * Set the input sensitivity
  */
  outportb( wADaddress  , bySettingCh1 );
  outportb( wADaddress+2,  0+4 ); // slct is high
  outportb( wADaddress+2, 11+4 ); // select Ch1 and clck is high
  outportb( wADaddress+2,  0+4 ); // slct is high

  outportb( wADaddress  , bySettingCh2 );
  outportb( wADaddress+2,  0 ); // slct is high
  outportb( wADaddress+2, 11 ); // select Ch2 and clck is high
  outportb( wADaddress+2,  0 ); // slct is high

  outportb( wADaddress  , 0xFF ); // Apply total power supply

} // SetSensitivity


word MeasureOneSampleBinary( byte byCh, word wMeasureDelay )
/* DESCRIPTION
 *   This routine measures one sample with the HANDYSCOPE and uses a delay
 *   before the sample. The data is returned in 12 bits binary
 * INPUT
 *   byte  byCh            the channel te measure from
 *   word  wMeasureDelay   the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 * OUTPUT
 *   return value          the measured value,    0 = max. neg. full scale
 *                                             4096 = max. pos. full scale
 */
{
  byte byChIndicator[4];
  word wCounter = 0;
  word wD0_D1;        // help variable to hold data bits D0 to D1
  word wD2_D6;        // help variable to hold data bits D2 to D6
  word wD7_D11;       // help variable to hold data bits D7 to D11
  byte byDummy;       // used to perform a dummy read operation

  switch ( byCh )
  {
    case CH1 : byChIndicator[0] = 7;
               byChIndicator[1] = 5;
               byChIndicator[2] = 0;
               byChIndicator[3] = 2;
               break;
    case CH2 : byChIndicator[0] = 3;
               byChIndicator[1] = 1;
               byChIndicator[2] = 4;
               byChIndicator[3] = 6;
               break;
  } // switch
 /*
  * read D7 .. D11
  */
  outportb( wADaddress+2, byChIndicator[0] );
  while ( wCounter < wMeasureDelay )
  {
    wCounter++;
  } // while
  wD7_D11 = inportb( wADaddress+1 );
  wD7_D11 = wD7_D11 & 0xF8;
  wD7_D11 = wD7_D11 << 4;
 /*
  * read D2..D6
  */
  outportb( wADaddress+2, byChIndicator[1] );
  wD2_D6 = inportb( wADaddress+1 );
  wD2_D6 = wD2_D6 & 0xF8;
  wD2_D6 = wD2_D6 >> 1;
 /*
  * dummy read
  */
  outportb( wADaddress+2, byChIndicator[2] );
  byDummy = inportb( wADaddress+1 );
 /*
  * read D0..D1
  */
  outportb( wADaddress+2, byChIndicator[3] );
  wD0_D1 = inportb( wADaddress+1 );
  wD0_D1 = wD0_D1 & 0x60;
  wD0_D1 = wD0_D1 >> 5;
 /*
  * switch power back on
  */
  outportb( wADaddress , 0xFF );

  return (wD0_D1 + wD2_D6 + wD7_D11 );
} // MeasureOneSampleBinary


float MeasureOneSampleInVolts( byte byCh, word wMeasureDelay, float rSens )
/* DESCRIPTION
 *   This routine measures one sample with the HANDYSCOPE and uses a delay
 *   before the sample. The data is returned in volts.
 * INPUT
 *   byte  byCh            the channel te measure from
 *   word  wMeasureDelay   the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 *   float rSens           The full scale sensitivity of the channel in volts
 * OUTPUT
 *   return value          the measured value in volts
 */
{
  float rHelp;
  rHelp = (float)(MeasureOneSampleBinary(CH2,wMeasureDelay)-(float)2048)/(float)2048*
          rSens;


  return ( rHelp );
} // MeasureOneSampleInVolts


void MeasureHandyScope( int nMeasureCount, word wMeasureDelay )
/* DESCRIPTION
 *   This routine measures a number of samples with the HANDYSCOPE
 *   and uses a delay for each sample. The data is stored binary.
 * INPUT
 *   int  nMeasureCount    the number of samples to be measured
 *   word wMeasureDelay    the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 * OUTPUT
 *   the sampled data is stored in a global array called wMeasurementArray
 */
{
  int  nSampleNumber; // holds the sample number

  nSampleNumber = -1;
  while ( nSampleNumber < nMeasureCount )
  {
    if ( nSampleNumber > -1 )
    {
      wMeasurementArray[CH2][nSampleNumber] = MeasureOneSampleBinary( CH2, wMeasureDelay );
      wMeasurementArray[CH1][nSampleNumber] = MeasureOneSampleBinary( CH1, wMeasureDelay );
    }
    nSampleNumber++;
  } // while

} // MeasureHandyScope

