/* File Ad_HS508.c */

/* System include files */

extern "C" {
 #include <dos.h>
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
 #include <string.h>
 #include <stdio.h>
 #include <graphics.h>

}

/* Custom include files */
#include "ad_hs508.h"
#include "addmath.h"
#include "globvars.h"
#include "types.h"
#include "const.h"


/* Global Variable Declarations */
// Following buffers specify where to find the CONTROL and DATA registers on the I/O map */
BYTE ContReg[] = {0, 4, 6};
BYTE DataReg[] = {1, 5, 7};

BYTE Level1_Con_Ad[]     = {0, 0};
BYTE Level2_Con_Ad[]     = {1, 1};
BYTE Trig_Niv_Con_Ad[]   = {2, 2};
BYTE Pga_Con_Ad[]        = {3, 3};
BYTE Data_Nul_Con_Ad[]   = {4, 4};
BYTE Trig_Reset_Con_Ad[] = {5, 5};

/* Variables for the DATA register ---> are being initialized */
BYTE Post_Lo_Reg        = 0;   // register 0
BYTE Post_Hi_Reg        = 0;   // register 1
BYTE TimeBase_Reg       = 0;   // register 3
BYTE Mode_Reg           = 0;   // register 4
BYTE Reset_Reg          = 0;   // register 5
BYTE Count_Reg          = 0;   // register 6
BYTE Write_TimeBase_Reg = 0;   // register 7

/* Register Contants Variables */
BYTE Level1_Reg_Ad[]     = {0, 0};
BYTE Level2_Reg_Ad[]     = {0, 0};
BYTE Trig_Niv_Reg_Ad[]   = {0, 0};
BYTE Pga_Reg_Ad[]        = {0, 0};
BYTE Trig_Reset_Reg_Ad[] = {0, 0};

int LPTAddress = LPT2ADDRESS;  /*!!!!!!!!!!!!!!!!!!!*/


int SwitchSetting[NUMCHANNELS][12] =
{
  {
    PGA1ATT1  ,  //  0.02 V  // just for compatibility
    PGA1ATT1  ,  //  0.04 V  // just for compatibility
    PGA1ATT1  ,  //  0.08 V  // just for compatibility
    PGA1ATT1  ,  //  0.20 V
    PGA1ATT1  ,  //  0.40 V
    PGA1ATT1  ,  //  0.80 V
    PGA1ATT10 ,  //  2    V
    PGA1ATT10 ,  //  4    V
    PGA1ATT10 ,  //  8    V
    PGA1ATT100,  // 20    V
    PGA1ATT100,  // 40    V
    PGA1ATT100   // 80    V
  },
  {
    PGA2ATT1   + PGA2ON,  //  0.02 V  // just for compatibility
    PGA2ATT1   + PGA2ON,  //  0.04 V  // just for compatibility
    PGA2ATT1   + PGA2ON,  //  0.08 V  // just for compatibility
    PGA2ATT1   + PGA2ON,  //  0.20 V
    PGA2ATT1   + PGA2ON,  //  0.40 V
    PGA2ATT1   + PGA2ON,  //  0.80 V
    PGA2ATT10  + PGA2ON,  //  2    V
    PGA2ATT10  + PGA2ON,  //  4    V
    PGA2ATT10  + PGA2ON,  //  8    V
    PGA2ATT100 + PGA2ON,  // 20    V
    PGA2ATT100 + PGA2ON,  // 40    V
    PGA2ATT100 + PGA2ON   // 80    V
  }
};

float SwitchValue[12] =
{
  0.02, 0.04, 0.08, 0.2, 0.4, 0.8, 2.0, 4.0, 8.0, 20.0, 40.0, 80.0
};

float DcOffsetGain = 0.57;

float ValueRef[3][12];
int   ValueDA[3][12];

/*
 * Local function declarations
 */
int  Clip          (int nLo, int nValue, int nHi);
void SetPGA        (BOOL bCouplingCh1, BOOL bCouplingCh2, long int lSetting);
void SetDA         (int nCh, float rValue);
BOOL EOC           (void);
void SetPostCount  (WORD wPostCount);
void SetTrigLevel  (HS508_MEASDEP* pMeasSettings);
void SetTrigBit    (void);
void SetTimeBase   (float* fFreq);
void ClearSRAM     (void);
void GoToSRAMPos   (WORD wSRAMPos);
void ReadSRAMData  (HS508_MEASDEP* pMeasSettings);
void MeasureHS508  (HS508_MEASDEP* pMeasSettings);
void ReadDigitalInputs(HS508_MEASDEP* pMeasSettings);
void WriteDigitalOutputs(BYTE byValue);

/*
 * Ad_HS508.c Local function definitions...
 */


void SelectRegister( BYTE byRegister )
/* DESCRIPTION
 *   This routine selects whether a TiePie Parallel Printer Interface (TPPPI)
 *   internal or external register is selected
 *   Use the predefined constants ExtRegister and IntRegister
 * INPUT PARAMETERS
 *   byRegister  indicates internal or external
 * OUTPUT PARAMETERS
 *   -
 */
{
  int LPTAddress_2 = LPTAddress + 2;

  asm {
    mov DX, LPTAddress_2
    mov AL, PPCONTROL
    out DX, AL

    mov DX, LPTAddress
    mov AL, byRegister
    out DX, AL

    mov DX, LPTAddress_2
    mov AL, (PPDEVSEL or PPCONTROL)
    out DX, AL

    mov AL, PPCONTROL and (255-PPDEVSEL)
    out DX, AL
  }  // asm
} // SelectRegister


void WriteToTPPPI( BYTE byAddress, BYTE byValue )
/* DESCRIPTION
 *   This routine writes a byte to the selected register in the HS508
 *   Before this routine is called, there has to be selected whether an
 *   internal or external register is used.
 * INPUT PARAMETERS
 *   byAddress  the address of the register
 *   byValue    the value that has to be written to the register
 * OUTPUT PARAMETERS
 *   -
 */
{
  int LPTAddress_2 = LPTAddress + 2;

  asm {
    mov DX, LPTAddress_2
    mov AL, PPCONTROL
    out DX, AL

    mov DX, LPTAddress
    mov AL, byAddress
    out DX, AL

    mov DX, LPTAddress_2
    mov AL, (PPADRSEL or PPCONTROL)
    out DX, AL
    and AL, (255-PPADRSEL)
    out DX, AL

    mov DX, LPTAddress
    mov AL, byValue
    out DX, AL

    mov DX, LPTAddress_2
    mov AL, (255-PPIOW) and PPCONTROL
    out DX, AL
    mov AL, (PPIOW or PPCONTROL)
    out DX, AL
  }  // asm
} // WriteToTPPPI


BYTE ReadFromTPPPI( BYTE byAddress )
/* DESCRIPTION
 *   This routine reads a byte from the register selected by byAddress
 *   through the TPPPI
 *   Before calling, the internal / external selection has to be made
 * INPUT PARAMETERS
 *   byAddress  the requested register
 * OUTPUT PARAMETERS
 *   returnvalue  the byte from the selected address
 */
{
  BYTE byLo, byHi;
  int LPTAddress_1 = LPTAddress + 1;
  int LPTAddress_2 = LPTAddress + 2;

  asm {
    mov DX, LPTAddress
    mov AL, byAddress
    out DX, AL

    mov DX, LPTAddress_2
    mov AL, (PPCONTROL or PPADRSEL)
    out DX, AL
    mov AL, PPCONTROL
    out DX, AL
    mov AL, PPCONTROL or PPIOR
    out DX, AL
    mov AL, PPCONTROL
    out DX, AL
    mov AL, (PPCONTROL or PPADRSEL)
    out DX, AL

    mov DX, LPTAddress
    mov AL, 1
    out DX, AL

    mov DX, LPTAddress_1
    in  AL, DX
    xor AL, 0x80
    shl AL, 1
    and AL, 0xF0
    mov byHi, AL

    mov DX, LPTAddress
    mov AL, 0
    out DX, AL

    mov DX, LPTAddress_1
    in  AL, DX
    xor AL, 0x80
    shr AL, 3
    and AL, 0x0F
    mov byLo, AL

    mov DX, LPTAddress_2
    mov AL, PPCONTROL
    out DX, AL
  } // asm
  return( byHi + byLo );
} // ReadFromTPPPI


int Clip( int nLo, int nValue, int nHi)
/* DECRIPTION
 *   Clip the parameter nValue btween nLo and nHi
 * INPUT PARAMETERS
 *   nLo           the lower limit
 *   nValue        the value to clip
 *   nHi           the higher limit
 * OUTPUT PARAMETERS
 *   returnvalue   the clipped value
 */
{
  if ( nValue > nHi )
  {
    nValue = nHi;
  }
  else
  {
    if ( nValue < nLo )
    {
      nValue = nLo;
    }
  }
  return nValue;
} // Clip


void SetPGA (BOOL bCouplingCh1, BOOL bCouplingCh2, long int lSetting)
/* DESCRIPTION:
 *   This routine converts the switch data and writes it to the hardware
 * INPUT PARAMETERS:
 *   bCouplingCh1 coupling setting Ch1
 *   bCouplingCh2 coupling setting Ch2
 *   lSetting     the other switches (attenuators)
 * OUTPUT PARAMETERS:
 *   -
 */
{
  const nSerialDataLength = 9;   // 10 bits to send
  const PGA_BLNK = 64;           // 0100 0000
  const PGA_SLCT = 16;           // 0001 0000

  BYTE S_Clck, S_Data;
  long int lHelp;
  int nCount;
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );

  if (!bCouplingCh1)
  {
    lSetting += PGA1ACDC;
  }
  if (!bCouplingCh2)
  {
    lSetting += PGA2ACDC;
  }
 /*
  * select the serial register of CONTROL REGISTER 2
  */
  WriteToTPPPI( ContReg[1], SERIAL_CON );
  lHelp =  512;  // this is 0010 0000 0000
  S_Clck = 0;
  S_Data = 0;
  WriteToTPPPI( DataReg[1], (BYTE)S_Data+S_Clck+PGA_BLNK );

  for (nCount = 0; nCount <= nSerialDataLength; nCount++)
  {
   /*
    * monitor each bit the value to send
    */
    if ((lSetting & lHelp) == 0)
    {
      S_Data = 0;
    }
    else
    {
      S_Data = 1;
    }
    S_Clck = 0;
    WriteToTPPPI( DataReg[1], (BYTE)S_Data+S_Clck+PGA_BLNK );
    S_Clck = 2;
    WriteToTPPPI( DataReg[1], (BYTE)S_Data+S_Clck+PGA_BLNK );
   /*
    * goto next bit
    */
    lHelp = lHelp >> 1;
  }
  WriteToTPPPI( DataReg[1], (BYTE)PGA_SLCT+PGA_BLNK );
  WriteToTPPPI( DataReg[1], (BYTE)PGA_BLNK );
 /*
  * Wait X msec for switching PGA
  */
  delay (PGA_SWITCHINGTIME);
} //SetPGA


void SetDA (int nCh, float rValue )
/* DESCRIPTION:
 *   This routine sets the DC offset level for a channel
 * INPUT PARAMETERS:
 *   nCh    indicates which channel to set
 *   rValue the value to set
 * OUTPUT PARAMETERS:
 *   -
 */
{
  const nSerialDataLength = 15;  // 16 bits to send
  const PGA_BLNK = 64;           // 0100 0000
  const AD_SLCT  =  8;           // 0000 1000

  BYTE S_Clck, S_Data;
  long int lHelp;
  int nData;
  int nCount;
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * select the serial register of CONTROL REGISTER 2
  */
  WriteToTPPPI( ContReg[1], SERIAL_CON );

  rValue = ((rValue/((2.8-0.3)/2))*512)+512;
  if (rValue > 1023)
  {
    rValue = 1023;
  }
  if (rValue < 0)
  {
    rValue = 0;
  }
  nData = (int)rValue;
  nData = (0x09 << 12) +   // 1001 .... .... ....
          (nCh  << 10) +   // .... xx.. .... ....
          nData;           // .... ..xx xxxx xxxx

  lHelp =  32768;  // this is 1000 0000 0000 0000
  S_Clck = 0;
  S_Data = 0;
  WriteToTPPPI( DataReg[1], (BYTE)((S_Data+S_Clck+AD_SLCT) | PGA_BLNK));
 /*
  * Send all 16 bits
  */
  for (nCount = 0; nCount <= nSerialDataLength; nCount++)
  {
   /*
    * monitor each bit the value to send
    */
    if ((nData & lHelp) == 0)
    {
      S_Data = 0;
    }
    else
    {
      S_Data = 1;
    }
    S_Clck = 0;
    WriteToTPPPI(DataReg[1],(BYTE)((S_Data+S_Clck+AD_SLCT)|PGA_BLNK));
    S_Clck = 2;
    WriteToTPPPI(DataReg[1],(BYTE)((S_Data+S_Clck+AD_SLCT)|PGA_BLNK));
   /*
    * goto next bit
    */
    lHelp = lHelp >> 1;
  }
  S_Clck = 0;
  WriteToTPPPI(DataReg[1],(BYTE)((S_Data+S_Clck+AD_SLCT)|PGA_BLNK));
  S_Clck = 2;
  WriteToTPPPI(DataReg[1],(BYTE)((S_Data+S_Clck+AD_SLCT)|PGA_BLNK));
  WriteToTPPPI(DataReg[1],(BYTE)PGA_BLNK);
} //SetDA


/* NAME:
 *   BOOL EOC( void )
 * DESCRIPTION:
 *   This routine reads the End Of Conversion bit
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   returnvalue is the End Of Conversion bit
 */
BOOL EOC( void )
{
  int nHelp;
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
  nHelp = (int) ReadFromTPPPI( 2 );
  nHelp = (int) ReadFromTPPPI( 2 );
 if ( (nHelp & 0x01) == 0x01 )
  {
    return( TRUE );
  }
  else
  {
    return( FALSE );
  }
} //EOC


/* NAME:
 *   void SetPostCount (WORD wPostCount)
 * DESCRIPTION:
 *   This routine sets the value for the post counter
 * INPUT PARAMETERS:
 *   wPostCount  contains the new value for the post counter
 * OUTPUT PARAMETERS:
 *   -
 */
void SetPostCount (WORD wPostCount)
{
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * split the post counter value in lower and higher byte
  */
  Post_Lo_Reg = LOBYTE( wPostCount );
  Post_Hi_Reg = HIBYTE( wPostCount );
 /*
  * select the POST-LO register and store the lower byte of the value into it
  */
  WriteToTPPPI( ContReg[0], (BYTE)POST_LO_CON );
  WriteToTPPPI( DataReg[0], (BYTE)Post_Lo_Reg );
 /*
  * select the POST-HI register and store the higher byte of the value into it
  */
  WriteToTPPPI( ContReg[0], (BYTE)POST_HI_CON );
  WriteToTPPPI( DataReg[0], (BYTE)Post_Hi_Reg );
}//SetPostCount



/* NAME:
 *   void SetTrigLevel( HS508_MEASDEP* pMeasSettings )
 * DESCRIPTION:
 *   This routine sets the trigger level
 * INPUT PARAMETERS:
 *   pMeasSettings Some instrument settings, including the trigger settings
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigLevel( HS508_MEASDEP* pMeasSettings )
{
  int nHelp, nCh;

 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );

  for (nCh = 0; nCh <= 1; nCh++)
  {
    if (pMeasSettings->TrigSlope == POS) // Positive Slope
    {
      nHelp = pMeasSettings->TrigLevel - pMeasSettings->TrigHysteresis;
      Level1_Reg_Ad[nCh] = Clip(0, nHelp, 255);
      Level2_Reg_Ad[nCh] = Clip(0, pMeasSettings->TrigLevel, 255);
      if (pMeasSettings->TrigSource == EXT)
      {
        Trig_Niv_Reg_Ad[nCh] = 1+16; // 0001 0001
      }
      else
      {
        Trig_Niv_Reg_Ad[nCh] = 5; // 0000 0101
      }
    }
    else // Negative Slope
    {
      nHelp = pMeasSettings->TrigLevel + pMeasSettings->TrigHysteresis;
      Level1_Reg_Ad[nCh] = Clip(0, nHelp, 255);
      Level2_Reg_Ad[nCh] = Clip(0, pMeasSettings->TrigLevel, 255);
      if (pMeasSettings->TrigSource == EXT)
      {
        Trig_Niv_Reg_Ad[nCh] = 0+16; // 0001 0000
      }
      else
      {
        Trig_Niv_Reg_Ad[nCh] = 6; // 0000 0110
      }
    }
    WriteToTPPPI( ContReg[nCh+1], Trig_Niv_Con_Ad[nCh] );
    WriteToTPPPI( DataReg[nCh+1], Trig_Niv_Reg_Ad[nCh] );

    WriteToTPPPI( ContReg[nCh+1], Level1_Con_Ad[nCh] );
    WriteToTPPPI( DataReg[nCh+1], Level1_Reg_Ad[nCh] );
    WriteToTPPPI( ContReg[nCh+1], Level2_Con_Ad[nCh] );
    WriteToTPPPI( DataReg[nCh+1], Level2_Reg_Ad[nCh] );
  }

} //SetTrigLevel


/* NAME:
 *   void SetTrigBit( void )
 * DESCRIPTION:
 *   This routine sets the trigger bit.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigBit ( void )
{
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
//  WriteToTPPPI( 0, (BYTE)7 );
//  WriteToTPPPI( 1, (BYTE)2 );
  WriteToTPPPI( ContReg[0], SELECTTRIG_CON );
  WriteToTPPPI( DataReg[0], (BYTE)2 );
} // SetTrigBit



void SetTimeBase (float* fFreq)
/* DESCRIPTION:
 *   This routine sets the timebase register. Some frequencies can be made
 *   directly by dividing the clock twice or five times. For other frequencies
 *   the 8253 has to be programmed with another value.
 * INPUT PARAMETERS:
 *   fFreq    the frequency to make
 * OUTPUT PARAMETERS:
 *   fFreq    the frequency that is made
 * CAVEAT
 *   This routine has several exit points
 */
{
  #define MaxFreq    25.0e+6 // max sampling frequency HS508
  #define FreqDiv2   12.5e+6 // max sampling frequency /   2
  #define FreqDiv5    5.0e+6 // max sampling frequency /   5
  #define FreqDiv10   2.5e+6 // max sampling frequency /  10
  #define FreqDiv500  5.0e+4 // max sampling frequency / 500
  #define FreqLoLim   8.0e+1 // min frequency to be generated with one 16-bit
                             // divider ( 5e+6 / 65536 )
  #define Div100      100    // used for divide by 100

  WORD wDivFactor;
  float fInputFreq;

 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * The three highest frequencies are generated by dividing the clock
  * oscillator by 1, by 2 or by 5, in the FPGA
  */
  if ( *fFreq > FreqDiv2 )
  {
    TimeBase_Reg = 0 + 32;                    // 0010 0000
    WriteToTPPPI( ContReg[0], TIMEBASE_CON );
    WriteToTPPPI( DataReg[0], TimeBase_Reg );
    *fFreq = MaxFreq;
    return;
  }
  if ( (*fFreq < FreqDiv2) && (*fFreq > FreqDiv5) )
  {
    TimeBase_Reg = 4 + 32;                    // 0010 0100
    WriteToTPPPI( ContReg[0], TIMEBASE_CON );
    WriteToTPPPI( DataReg[0], TimeBase_Reg );
    *fFreq = FreqDiv2;
    return;
  }
  if ( (*fFreq < FreqDiv5) && (*fFreq > FreqDiv10) )
  {
    TimeBase_Reg = 8 + 32;                    // 0010 1000
    WriteToTPPPI( ContReg[0], TIMEBASE_CON );
    WriteToTPPPI( DataReg[0], TimeBase_Reg );
    *fFreq = FreqDiv5;
    return;
  }
 /*
  * The rest of the frequencies are generated using the 8253, which is
  * clocked at MaxFreq / 5 ( = 5MHz)
  */
  fInputFreq = FreqDiv5; // this is the oscillator frequency
 /*
  * a special division factor has to be calculated
  */
  if (*fFreq > FreqDiv5)
  {
    *fFreq = FreqDiv5;
  }
  if (*fFreq < FreqLoLim)
  {
    fInputFreq = FreqDiv500;
  }
  wDivFactor = (WORD)Round (double(fInputFreq/(*fFreq)));
  *fFreq     = fInputFreq/wDivFactor;
 /*
  * Initialize timer IC 8254, control word A1, A0=11
  */
  TimeBase_Reg = 3 + 32;          // 0010 0011
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg);
  WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)54); // 00 11 011 0
               /*
                *  00 [=select counter 0]
                *  11 [=write LSBYTE first]
                * 011 [=mode 3, divide by n counter]
                *   0 [=16-bits counter]
                */
 /*
  * Load DivFactor
  */
  TimeBase_Reg = 0 + 32;          // 0010 0000
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON       );
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg       );
  WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)LOBYTE(wDivFactor) );
  WriteToTPPPI( DataReg[0], (BYTE)HIBYTE(wDivFactor) );

  if (*fFreq < FreqLoLim)
  {
   /*
    * Extra divider needed:
    * Initialize timer IC 8254, control word A1,A0=11
    */
    TimeBase_Reg = 3 + 32;          // 0010 0011
    WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON);
    WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg);
    WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON);
    WriteToTPPPI( DataReg[0], (BYTE)118); // 01 11 011 0
                 /*
                  *  01 [=select counter 1]
                  *  11 [=write LSBYTE first]
                  * 011 [=mode 3, divide by n counter]
                  *   0  [=16-bits counter]
                  */
   /*
    * Load DivFactor
    */
    TimeBase_Reg = 1 + 32;       // 0010 0001
    WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON       );
    WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg       );
    WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON );
    WriteToTPPPI( DataReg[0], (BYTE)LOBYTE(Div100)     );
    WriteToTPPPI( DataReg[0], (BYTE)HIBYTE(Div100)     );
  }
 /*
  * Select Clock Input
  */
  if (*fFreq < FreqLoLim)
  {
    TimeBase_Reg = 16+32; // Select clck input b
  }
  else
  {
    TimeBase_Reg = 12+32; // Select clck input a
  }
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg);
} //SetTimeBase


void SetCalOutput(float* fFreq)
/* DESCRIPTION:
 *   This routine sets the generator output. For the frequencies
 *   the 8253 has to be programmed with a value.
 * INPUT PARAMETERS:
 *   fFreq    the frequency to make
 * OUTPUT PARAMETERS:
 *   fFreq    the frequency that is made
 */
{
  #define MaxFreq     1.0e+5 // max frequency generator output
  #define MinFreq     1.0e+2 // min frequency generator output
  #define ClckFreq    5.0e+6 // clock frequency for timer chip

  WORD wDivFactor;
  float fInputFreq;

 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * The frequencies are generated using the 8253, which is
  * clocked at ClckFreq ( = 5MHz)
  */
  fInputFreq = ClckFreq; // this is the oscillator frequency
 /*
  * a division factor has to be calculated
  */
  if (*fFreq > MaxFreq)
  {
    *fFreq = MaxFreq;
  }
  if (*fFreq < MinFreq)
  {
    *fFreq = MinFreq;
  }
  wDivFactor = (WORD)Round (double(fInputFreq/(*fFreq)));
  *fFreq     = fInputFreq/wDivFactor;
 /*
  * Initialize timer IC 8254, control word A1, A0=11
  */
  TimeBase_Reg = 3 + 32;          // 0010 0011
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg);
  WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)182); // 10 11 011 0
               /*
                *  10 [=select counter 2]
                *  11 [=write LSBYTE first]
                * 011 [=mode 3, divide by n counter]
                *   0 [=16-bits counter]
                */
 /*
  * Load DivFactor
  */
  TimeBase_Reg = 2 + 32;          // 0010 0010
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON       );
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg       );
  WriteToTPPPI( ContReg[0], (BYTE)WRITE_TIMEBASE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)LOBYTE(wDivFactor) );
  WriteToTPPPI( DataReg[0], (BYTE)HIBYTE(wDivFactor) );

  TimeBase_Reg = 3 + 32;          // 0010 0011
  WriteToTPPPI( ContReg[0], (BYTE)TIMEBASE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)TimeBase_Reg);

} //SetTimeBase


/* NAME:
 *   void ClearSRAM (void)
 * DESCRIPTION:
 *   This routine clears the SRAM
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void ClearSRAM (void)
{
  #define HS508MemSize 32 * 1024
  #define MaxFreq      25.0e+6
  int nCh;
  float fFreq;

 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * disable clock
  */
  WriteToTPPPI( ContReg[0], MODE_CON );
  WriteToTPPPI( DataReg[0], 0 );
 /*
  * Clear the whole record
  */
  SetPostCount(HS508MemSize + 1);
 /*
  * Set the SRAM clear bit
  */
  for (nCh = 0; nCh <= 1; nCh++)
  {
   /*
    * set Data zero register
    */
    WriteToTPPPI( ContReg[nCh+1], DATAZERO_CON );
    WriteToTPPPI( DataReg[nCh+1], (BYTE)1); // 0000 0001
  }
 /*
  * Set fastest TimeBase
  */
  fFreq = MaxFreq;
  SetTimeBase (&fFreq);
 /*
  * Set Clck Source : Sample clock off
  *                   Internal clock
  *                   Data from ADC to SRAM and PC-bus
  *                   Conversion clock off
  */
  WriteToTPPPI( ContReg[0], MODE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)6);  // 0000 0110
 /*
  * Reset Post Counter and Reset Address Counter
  */
  WriteToTPPPI( ContReg[0], RESET_CON);
  WriteToTPPPI( DataReg[0], (BYTE)0);    // 0000 0000
 /*
  * Set Clck Source : Sample clock on
  *                   Internal clock
  *                   Data from ADC to SRAM and PC-bus
  *                   Conversion clock off
  */
  WriteToTPPPI( ContReg[0], MODE_CON);
  WriteToTPPPI( DataReg[0], (BYTE)7);  // 0000 0111
 /*
  * From now on, it takes 1.31072 msec to clear the SRAM, the process stops
  * automatically
  */
} // ClearSRAM



void GoToSRAMPos (WORD wSRAMPos)
/* DESCRIPTION:
 *
 * INPUT PARAMETERS:
 *   wSRAMPos  to position in the SRAM to go to
 * OUTPUT PARAMETERS:
 *   -
 */
{
  #define MaxFreq 25.0e+6
  float fFreq;

 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * disable clock
  */
  WriteToTPPPI( ContReg[0], MODE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0  );
 /*
  * Load postcounter with the address
  */
  SetPostCount( wSRAMPos );
 /*
  * Select fastest timebase
  */
  fFreq = MaxFreq;
  SetTimeBase (&fFreq);
 /*
  * Reset post counter
  */
  WriteToTPPPI( ContReg[0], RESET_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0   );
 /*
  * Select immediate triggering
  */
  WriteToTPPPI( ContReg[0], SELECTTRIG_CON );
  WriteToTPPPI( DataReg[0], (BYTE)2        );
 /*
  * Select clock source and turn on clock
  */
  WriteToTPPPI( ContReg[0],MODE_CON );
  WriteToTPPPI( DataReg[0],(BYTE)3  ); // 0000 0011
 /*
  * Wait until TrigBit == '1'
  */
  do
  {
   // just do nothing, wait
  }
  while ( !EOC() );
 /*
  * Switch clock off
  */
  WriteToTPPPI( ContReg[0], MODE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0  );

}  //GoToSRAMPos



/* NAME:
 *   void ReadSRAMData (HS508_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine reads the SRAM data to the data buffer in the PC RAM
 * INPUT PARAMETERS:
 *   pMeasSettings  contains measurement instrument settings
 * OUTPUT PARAMETERS:
 *   -
 */
void ReadSRAMData (HS508_MEASDEP* pMeasSettings)
{
  int nSampleIndex;
  int nSampleIndexMax;
  long int lHelp, lCounter_Lo, lCounter;
  WORD wHelp;
  int LPTAddress_1 = LPTAddress + 1;
  int LPTAddress_2 = LPTAddress + 2;
  unsigned int nADvalue1,
               nADvalue2;

 /*
  * Switch clock off
  */
  WriteToTPPPI( ContReg[0], MODE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0  );
 /*
  * Read Address Counter
  */
  lCounter_Lo = ReadFromTPPPI(0);
  lCounter = (long)ReadFromTPPPI(1)*256 + lCounter_Lo;
  lCounter = (lCounter & 32767 );
  if (pMeasSettings->TrigSource == NO_TRIG)
  {
    lHelp = lCounter - (pMeasSettings->NumPostSamples + pMeasSettings->NumPreSamples) - 6;
  }
  else
  {
    lHelp = lCounter - (pMeasSettings->NumPostSamples + pMeasSettings->NumPreSamples) - 7;
  }
  if (lHelp < 0)
  {
    lHelp += 32767;
  }
  GoToSRAMPos (lHelp);
 /*
  * Setup for reading data
  */
  asm {
    MOV DX,LPTAddress_2
    MOV AL,PPCONTROL
    OUT DX,AL
   /*
    *Select External register
    */
    MOV DX,LPTAddress
    MOV AL,ExtRegister
    OUT DX,AL

    MOV DX,LPTAddress_2
    MOV AL,(PPDEVSEL OR PPCONTROL)
    OUT DX,AL
    MOV AL,PPCONTROL
    OUT DX,AL
   /*
    * Load address=0 with data=5 ext device (=inc address)
    */
    MOV DX,LPTAddress
    MOV AL,0 // Address
    OUT DX,AL

    MOV DX,LPTAddress_2
    MOV AL,(PPADRSEL OR PPCONTROL)
    OUT DX,AL
    MOV AL,PPCONTROL
    OUT DX,AL

    MOV DX,LPTAddress
    MOV AL,5 // Data
    OUT DX,AL

    MOV DX,LPTAddress_2
    MOV AL,((255-PPIOW) AND PPCONTROL)
    OUT DX,AL
    MOV AL,PPCONTROL
    OUT DX,AL
  } // asm

  nSampleIndexMax = pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples;
  for ( nSampleIndex = 0;
        nSampleIndex < nSampleIndexMax;
        nSampleIndex++
      )
  {
    asm {
    /*
     *  Read AD Data CH 1
     *  select address
     */
     MOV DX,LPTAddress
     MOV AL,4 // Address 4 is active
     OUT DX,AL

     MOV DX,LPTAddress_2
     MOV AL,PPADRSEL+PPIOR+PPCONTROL
     OUT DX,AL
     MOV AL,PPADRSEL OR PPCONTROL
     OUT DX,AL
    /*
     *  LoData = ( (inportb(LPTAddress+1) << 5) & 0x0F00 );
     */
     MOV DX,LPTAddress_1
     IN  AL,DX
     SHL AX,5
     AND AX,0x0F00
     MOV CX,AX

     MOV DX,LPTAddress
     MOV AL,1 // Data Select Hi Byte
     OUT DX,AL
    /*
     *  HiData = ( (inportb(LPTAddress+1) << 9) & 0xF000 );
     */
     MOV DX,LPTAddress_1
     IN  AL,DX
     SHL AX,9
     AND AX,0xF000
     OR  CX,AX
     MOV nADvalue1,CX
    /*
     *  Read AD Data CH 2
     *  select adres
     */
     MOV DX,LPTAddress
     MOV AL,6
     OUT DX,AL

     MOV DX,LPTAddress_2
     MOV AL,PPADRSEL OR PPIOR OR PPCONTROL
     OUT DX,AL
     MOV AL,PPADRSEL OR PPCONTROL
     OUT DX,AL
    /*
     *  LoData = ( (inportb(LPTAddress+1) << 5) & 0x0F00 );
     */
     MOV DX,LPTAddress_1
     IN  AL,DX
     SHL AX,5
     AND AX,0x0F00
     MOV CX,AX

     MOV DX,LPTAddress
     MOV AL,1 // Data Select Hi Byte
     OUT DX,AL
    /*
     *  LoData = ( (inportb(LPTAddress+1) << 9) & 0xF000 );
     */
     MOV DX,LPTAddress_1
     IN  AL,DX
     SHL AX,9
     AND AX,0xF000
     OR  CX,AX
     MOV nADvalue2,CX
    /*
     *  increase address counter
     *  BEWARE: At LPTAddress is already "1", or Address=1
     */
     MOV DX,LPTAddress_2
     MOV AL,(255-PPIOW) AND PPCONTROL // write dummy byte
     OUT DX,AL

     MOV AL,PPADRSEL OR PPCONTROL
     OUT DX,AL
    } // asm

    *(pGlobalRAMDataCH1+nSampleIndex) = nADvalue1;
    *(pGlobalRAMDataCH2+nSampleIndex) = nADvalue2;

  } // for
 /*
  * Reset Trigger Ch1 and Ch2
  */
  WriteToTPPPI( ContReg[1], TRIGRESET_CON);
  WriteToTPPPI( DataReg[1], (BYTE)0);
  WriteToTPPPI( ContReg[2], TRIGRESET_CON);
  WriteToTPPPI( DataReg[2], (BYTE)0);

  ClearSRAM (); // For next measurement

} //ReadSRAMData



/* NAME:
 *   void MeasureHS508 (HS508_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine sets the HS508 to perform one measurement on both
 *   channels
 * INPUT PARAMETERS:
 *   pMeasSettings   instrument and measurement settings
 * OUTPUT PARAMETERS:
 *   -
 */
void MeasureHS508 (HS508_MEASDEP* pMeasSettings)
{
  #define MaxFreq 25.0e+6
  BYTE byHelp;
  float fFreq;

  SetTrigLevel( pMeasSettings );
  SetPostCount(pMeasSettings->NumPostSamples);
 /*
  * Select TPPPI external registers
  */
  SelectRegister( ExtRegister );
 /*
  * set sampling frequency
  */
  if ( (pMeasSettings->SampleFreq > MaxFreq  ) &&
       (pMeasSettings->MeasurementMode == CH1)
     )
  {
   /*
    * select 50 Msamples/sec, only possible at ch1
    */
    fFreq = MaxFreq;
    SetTimeBase( &fFreq );
    WriteToTPPPI( ContReg[1], (BYTE)DATAZERO_CON );
    WriteToTPPPI( DataReg[1], (BYTE)2            ); // normal clock for Ch1
    WriteToTPPPI( ContReg[2], (BYTE)DATAZERO_CON );
    WriteToTPPPI( DataReg[2], (BYTE)0            ); // inverted clock for Ch2
  }
  else
  {
   /*
    * all other settings
    */
    SetTimeBase( &(pMeasSettings->SampleFreq) );
    WriteToTPPPI( ContReg[1], (BYTE)DATAZERO_CON );
    WriteToTPPPI( DataReg[1], (BYTE)2            ); // normal clock for Ch1
    WriteToTPPPI( ContReg[2], (BYTE)DATAZERO_CON );
    WriteToTPPPI( DataReg[2], (BYTE)2            ); // normal clock for Ch2
  }
 /*
  * Reset Post Counter and Address Counter
  */
  WriteToTPPPI( ContReg[0], RESET_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0 );
 /*
  * disable clocks
  */
  WriteToTPPPI( ContReg[0], MODE_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0 );
 /*
  * Set the trigger source
  */
  WriteToTPPPI( ContReg[0], (BYTE)SELECTTRIG_CON );
  switch (pMeasSettings->TrigSource)
  {
    case CH1:       WriteToTPPPI( DataReg[0], (BYTE)0 );
                    break;
    case CH2:       WriteToTPPPI( DataReg[0], (BYTE)1 );
                    break;
    case CH1ANDCH2: WriteToTPPPI( DataReg[0], (BYTE)3 );
                    break;
    case CH1ORCH2:  WriteToTPPPI( DataReg[0], (BYTE)4 );
                    break;
    case EXT:       WriteToTPPPI( DataReg[0], (BYTE)0 );
                    break;
    case NO_TRIG:   WriteToTPPPI( DataReg[0], (BYTE)2 );
                    break;
    default:        WriteToTPPPI( DataReg[0], (BYTE)2 );
                    break;
  } /* switch */
 /*
  * Reset Post Counter and Address Counter
  */
  WriteToTPPPI( ContReg[0], RESET_CON );
  WriteToTPPPI( DataReg[0], (BYTE)0 );
 /*
  * Let ADC data pass through
  */
  WriteToTPPPI( ContReg[1], DATAZERO_CON );
  WriteToTPPPI( DataReg[1], (BYTE)0 );
  WriteToTPPPI( ContReg[2], DATAZERO_CON );
  WriteToTPPPI( DataReg[2], (BYTE)0 );
/*
 * Reset Post Counter and Address Counter
 */
 WriteToTPPPI( ContReg[0], RESET_CON );
 WriteToTPPPI( DataReg[0], (BYTE)0 );
/*
 * Reset trig Ch1 and Ch2
 */
 WriteToTPPPI( ContReg[1], TRIGRESET_CON );
 WriteToTPPPI( DataReg[1], (BYTE)0 );
 WriteToTPPPI( ContReg[2], TRIGRESET_CON );
 WriteToTPPPI( DataReg[2], (BYTE)0 );
/*
 * Set Clck SRAM
 */
 WriteToTPPPI( ContReg[0], MODE_CON );
 WriteToTPPPI( DataReg[0], (BYTE)15 );

} //MeasureHS508



void SetEepromSdaScl( BYTE SDA, BYTE SCL )
{
  BYTE SclSdaByte = 4 + 8;      // 0000 1100
  if (SCL==1)
  {
    SclSdaByte |= 1;       // 0000 0001
  }
  else
  {
    SclSdaByte &= (255-1); // 1111 1110
  }
  if (SDA==1)
  {
    SclSdaByte |= 2;       // 0000 0010
  }
  else
  {
    SclSdaByte &= (255-2); // 1111 1101
  }
 /*
  * Select a TPPPI internal register
  */
  SelectRegister( IntRegister );
 /*
  * And write the data
  */
  WriteToTPPPI( 0, EEPROMREG  );
  WriteToTPPPI( 1, SclSdaByte );

} // SetEepromSdaScl


void SetEepromSdaSclRead( BYTE *SDA, BYTE SCL )
{
  BYTE SclSdaByte = 0 + 8;      // 0000 1000
  if (SCL==1)
  {
    SclSdaByte |= 1;       // 0000 0001
  }
  else
  {
    SclSdaByte &= (255-1); // 1111 1110
  }
 /*
  * Select a TPPPI internal register
  */
  SelectRegister( IntRegister );
 /*
  * And write the data
  */
  WriteToTPPPI( 0, EEPROMREG  );
  WriteToTPPPI( 1, SclSdaByte );

  outportb( LPTAddress+2, PPCONTROL);
  outportb( LPTAddress+0, 0);
  outportb( LPTAddress+2, PPCONTROL | PPADRSEL);
  outportb( LPTAddress+2, PPCONTROL & (255-PPADRSEL));

  *SDA = (((inportb(LPTAddress+1) >> 7) & 0x1) ^ 0x01);

} // SetEepromSdaSclRead


void StaDataEeprom( BYTE ReadWrite )
{
  #define EepromAddress 160   // the i2c eeprom address
  int  n, m;
  BYTE byInBit, byOutBit, byEepromData;

  SetEepromSdaScl( 1, 1 );
  SetEepromSdaScl( 0, 1 );
  SetEepromSdaScl( 0, 0 );

  byEepromData = EepromAddress + ReadWrite;

  m = 128;
  for ( n = 7; n >= 0; n-- )
  {
    if (byEepromData - m >= 0)
    {
      byEepromData -= m;
      byOutBit = 1;
    }
    else
    {
      byOutBit = 0;
    }
    m = m >> 1;

    SetEepromSdaScl( byOutBit, 0 );
    SetEepromSdaScl( byOutBit, 1 );
    SetEepromSdaScl( byOutBit, 0 );
  }
 /*
  * Acknowledge
  */
  SetEepromSdaSclRead( &byInBit, 0 );
  SetEepromSdaSclRead( &byInBit, 1 );
  SetEepromSdaScl    ( 0       , 0 );

} // StaDataEeprom


void StoDataEeprom ( void )
{
  SetEepromSdaScl( 0, 1 );
  SetEepromSdaScl( 1, 1 );
} // StoDataEeprom


void WriteDataEeprom( BYTE byEepromData )
{
  int n, m;
  BYTE byInBit, byOutBit;

  m = 128;
  for ( n = 7; n >= 0; n-- )
  {
    if (byEepromData - m >= 0)
    {
      byEepromData -= m;
      byOutBit = 1;
    }
    else
    {
      byOutBit = 0;
    }
    m = m >> 1;

    SetEepromSdaScl( byOutBit, 0 );
    SetEepromSdaScl( byOutBit, 1 );
    SetEepromSdaScl( byOutBit, 0 );
  }
 /*
  * Acknowledge
  */
  SetEepromSdaSclRead( &byInBit, 0 );
  SetEepromSdaSclRead( &byInBit, 1 );
  SetEepromSdaScl    ( 0       , 0 );

} // WriteDataEeprom


void ReadDataEeprom( BYTE *byEepromData )
{
  int  index;
  BYTE byInBit;

  *byEepromData = 0;
  for (index = 7; index >=0; index--)
  {
    SetEepromSdaSclRead( &byInBit, 0);
    SetEepromSdaSclRead( &byInBit, 1);
    *byEepromData += (byInBit << index);
    SetEepromSdaSclRead( &byInBit, 0);
  }//for
 /*
  * Acknowledge
  */
  SetEepromSdaScl( 1, 0 );
  SetEepromSdaScl( 1, 1 );
  SetEepromSdaScl( 1, 0 );
} // ReadDataEeprom

void ReadDataFromEeprom( BYTE byAddress, BYTE *byData )
{
  StaDataEeprom(0);
  WriteDataEeprom(byAddress);
  StoDataEeprom();

  StaDataEeprom(1);
  ReadDataEeprom(&*byData);
  StoDataEeprom();
} // ReadDataFromEeprom


void ReadEepromOffset( int nCh, int nSetting )
{
  int  nValue;
  float fValue;
  BYTE byHi, byLo;
  int  nAddress;

  switch (nCh)
  {
    case CHANNEL1: nAddress = EEPROMADROFFSETCH1 + nSetting*2;
                   break;
    case CHANNEL2: nAddress = EEPROMADROFFSETCH2 + nSetting*2;
                   break;
    case CHANNEL3: nAddress = EEPROMADROFFSETCH3 + nSetting*2;
                   break;
  } //switch
  ReadDataFromEeprom( nAddress, &byLo );
  ReadDataFromEeprom( nAddress+1, &byHi );
  nValue = (byHi << 8) + byLo;
  fValue = nValue-512;
  fValue = fValue / 512;
  ValueRef[nCh][nSetting] = ((2.8-0.3)/2)*fValue;
} // ReadEepromOffset


void ReadEepromGain( int nCh, int nSetting )
{
  unsigned int  nValue;
  BYTE byLoValue,
       byHiValue;
  int  nAddress;

  switch (nCh)
  {
    case CHANNEL1 : nAddress = EEPROMADRGAINCH1 + nSetting*2;
                    break;
    case CHANNEL2 : nAddress = EEPROMADRGAINCH2 + nSetting*2;
                    break;
    case CHANNEL3 : nAddress = EEPROMADRGAINCH3 + nSetting*2;
                    break;
  } // switch
  ReadDataFromEeprom(nAddress  , &byLoValue);
  ReadDataFromEeprom(nAddress+1, &byHiValue);
  nValue = (byHiValue << 8) + byLoValue;
  ValueDA[nCh][nSetting] = nValue;
} // ReadEEpromGain


BOOL IsHardwarePresent( void )
/* DESCRIPTION
 *   This routine checks whether the HS508 hardware is present
 * INPUT PARAMETERS
 *   -
 * OUTPUT PARAMETERS
 *   return value   TRUE  = hardware is     present
 *                  FALSE = hardware is not present
 */
{
  int index;
  int nValue;
  BYTE byEepromData;

  for ( index=1; index<=3; index++)
  {
    switch (index)
    {
      case 1 : LPTAddress = LPT1ADDRESS;
               break;
      case 2 : LPTAddress = LPT2ADDRESS;
               break;
      case 3 : LPTAddress = LPT3ADDRESS;
               break;
    } // switch
    ReadDataFromEeprom( EEPROMTYPEID, &byEepromData );
    if (byEepromData == IDDEVICE)
    {
      return( TRUE );
    } //if
  } // for
  return( FALSE );
} // IsHardwarePresent


long int GetSerialNumber( void )
/* DESCRIPTION
 *   This routine reads the HS508 serial number from the EEPROM
 * INPUT PARAMETERS
 *   -
 * OUTPUT PARAMETERS
 *   return value   The HS508 serial number
 */
{
  BYTE byLoSerialNr,
       byMiSerialNr,
       byHiSerialNr;

       if ( IsHardwarePresent )
       {
         ReadDataFromEeprom( EEPROMSERIALNR1+1, &byLoSerialNr );
         ReadDataFromEeprom( EEPROMSERIALNR1+1, &byMiSerialNr );
         ReadDataFromEeprom( EEPROMSERIALNR1+1, &byHiSerialNr );
         return( (byLoSerialNr) + (byLoSerialNr<<8) + (byLoSerialNr<<16) );
       }
       else
       {
         return( -1 );
       }
} // GetSerialNumber

void ReadCalibrateData( void )
/* DESCRIPTION
 *  This routine reads all calibration factors from the eeprom
 * INPUT PARAMETERS
 *  -
 * OUTPUT PARAMTERS
 *  -
 */
{
  int index, nCh;

  for (nCh=CHANNEL1; nCh<=CHANNEL3; nCh++)
  {
    for (index=0; index<=11; index++)
    {
      ReadEepromGain( nCh, index );
    } // for
    for (index=0; index<=11; index++)
    {
      ReadEepromOffset( nCh, index );
    } // for
  } // for
} // ReadCalibrateData


void CalibrateInstrument( void )
/* DESCRIPTION
 *   This routine calibrates the HS508
 * INPUT PARAMETERS
 *   -
 * OUTPUT PARAMETERS
 *   -
 */
{
  ReadCalibrateData();
} // CalibrateInstrument


