/*
 * File AD_HS508.h (HS508 specific constants)
 */

#ifndef __AD_HS508_H
#define __AD_HS508_H


/*
 * System include files
 */
#include "_defs.h"

/*
 * Custom include files
 */
#include "const.h"
#include "types.h"

#define PPCONTROL    4 // 0100

#define PPDEVSEL     1 // 0000 0001  STROBE
#define PPADRSEL     2 // 0000 0010  AUTO_FDXT
#define PPIOW        4 // 0000 0100  INIT
#define PPIOR        8 // 0000 1000  SLCT_IN

#define PIBIREG      0
#define PIEXTCLCKREG 1
#define PIRELAISREG  2
#define TIMERREG     3
#define EEPROMREG    4
#define OUTREG       5

/*
 * Address constants within the HS508 eeprom
 */
#define EEPROMTYPEID          1 // identify of devic at parallel port
#define EEPROMSERIALNR1       5 // type identification: 1 byte serial nr 3 bytes
#define EEPROMCALDATETIME    10 // year 2 bytes month 1 byte, day 1 byte hour 1 byte min 1 byte
#define EEPROMADRGAINCH1     50 // 12 gain settings, each 2 bytes
#define EEPROMADRGAINCH2     75
#define EEPROMADRGAINCH3    100
#define EEPROMADROFFSETCH1  125 // 12 offset settings, each 2 bytes
#define EEPROMADROFFSETCH2  150
#define EEPROMADROFFSETCH3  175
#define EEPROMSERIALNR2     200 // copy of serial nr

#define ExtRegister         0
#define IntRegister         0+16

#define IDDEVICE            170 // this code is placed in each eeprom
#define HSDEVICETYPEID      2   // HS508


/*
 * Wait after switching PGA
 */
#define PGA_SWITCHINGTIME	     10

/*
 * Values for the CONTROL registers in CONTROL REGISTER 1
 */
#define POST_LO_CON		0
#define POST_HI_CON		1
#define TIMEBASE_CON		2
#define MODE_CON		3
#define RESET_CON		4
#define COUNT_CON		5
#define WRITE_TIMEBASE_CON	6
#define SELECTTRIG_CON          7

/*
 * Values for the CONTROL registers in CONTROL REGISTER 2 and 3
 */
#define LEVEL1_CON		0
#define LEVEL2_CON		1
#define TRIGNIV_CON             2
#define SERIAL_CON 		3
#define DATAZERO_CON		4
#define TRIGRESET_CON		5

#define COUPLEAC		TRUE
#define COUPLEDC		FALSE


/*
 * Possible input voltages
 */
#define PGA20MVOLT		 0
#define PGA40MVOLT		 1
#define PGA80MVOLT               2
#define PGA200MVOLT		 3
#define PGA400MVOLT		 4
#define PGA800MVOLT              5
#define PGA2VOLT                 6
#define PGA4VOLT                 7
#define PGA8VOLT                 8
#define PGA20VOLT                9
#define PGA40VOLT               10
#define PGA80VOLT               11

#define PGA2ATT1                 1
#define PGA2ATT10                2
#define PGA2ATT100               4
#define PGA2ACDC                 8
#define PGA1ACDC                16
#define PGA1ATT1                32
#define PGA1ATT10               64
#define PGA1ATT100             128
#define PGA1ON                 256
#define PGA2ON                 512


#define LPT1ADDRESS            0x378
#define LPT2ADDRESS            0x278
#define LPT3ADDRESS            0x3BC

extern int  Ad_PortAddress;
extern int SwitchSetting[NUMCHANNELS][12];

extern float SwitchValue[12];
extern float ValueRef[3][12];
extern int   ValueDA[3][12];
extern float DcOffsetGain;

#endif