/*
 * File GlobVars.c ---> Global Variable Definitions
 */

/*
 * System include files
 */
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>

/*
 * Custom include files
 */
 #include "const.h"
 #include "globvars.h"


 #define TEXT_CH12SET_AUTO       "Auto ranging"

/*
 * Volt/Div predefines
 */
 const char* VoltDivBuffer[MAXVOLTDIVSTAND] = {   "5mV",
                                                 "10mV",
                                                 "20mV",
                                                 "50mV",
                                                "100mV",
                                                "200mV",
                                                 "0.5V",
                                                 "1.0V",
                                                 "2.0V",
                                                 "5.0V",
                                                "10.0V",
                                                "20.0V",
                                                TEXT_CH12SET_AUTO
                                              };


/*
 * Timebase predefines
 */
 const char* TimeDivBuffer[MAXTIMEDIVSTAND] = {  "20MHz",
                                                 "10MHz",
                                                  "4MHz",
                                                  "2MHz",
                                                  "1MHz",
                                                "500kHz",
                                                "200kHz",
                                                "100kHz",
                                                 "50kHz",
                                                 "10kHz",
                                                  "5kHz",
                                                  "1kHz",
                                                 "500Hz",
                                                 "100Hz",
                                                  "50Hz",
                                                  "20Hz",
                                                  "10Hz",
                                                   "5Hz"
                                              };

const char* PreSampBuffer[MAXSTANDPRESAMP] = {   "0%",  "5%", "10%", "15%",
                                                "20%", "25%", "30%", "35%",
                                                "40%", "45%", "50%", "55%",
                                                "60%", "65%", "70%", "75%",
                                                "80%", "85%", "90%", "95%",
                                               "100%"
                                             };
const char* TimeMagnBuffer[MAXSTANDTIMEMAGN] = { "0.5*",  "1*",  "2*",
                                                   "3*",  "4*",  "5*",
                                                   "6*",  "8*", "10*",
                                                  "12*", "15*", "16*",
                                                  "20*", "50*", "100*"
                                               };
const char* pUnitBuffer[MAXUNITSTAND] = {  "V",
                                           "A",
                                          "F",
                                          "C",
                                           "W",
                                           "%",
                                           "M",
                                          "Kg",
                                           "N",
                                           "C",
                                           "B",
                                          "Hz"
                                        };

const char* TimeOutBuffer [MAXTIMEOUTSTAND] = {    "0*",    "1*",    "2*",
                                                   "5*",   "10*",   "20*",
                                                  "50*",  "100*",  "200*",
                                                 "500*", "1000*", "2000*",
                                                "5000*", "INFINITE"
                                              };

const char* HysteresisBuffer[MAXHYSSTAND] = {     "0 DIV", "0.125 DIV",
                                              "0.250 DIV", "0.500 DIV",
					       "0.75 DIV",  "1.00 DIV",
                                               "1.25 DIV",  "1.50 DIV",
                                               "1.75 DIV",  "2.00 DIV",
                                               "2.25 DIV",  "2.50 DIV"
                                            };

const char* NumSampBuffer [MAXSAMPSTAND] = {  "1024 SAMP",  "2048 SAMP",
                                              "4096 SAMP",  "8192 SAMP",
                                             "16384 SAMP", "32767 SAMP"
                                           };

char* pTriggerTextBuffer [NUMTRIGPOS] = {         "CH1", // trig at ch1
                                                  "CH2", // trig at ch2
                                          "CH1 AND CH2", // trig at ch1 and ch2
					     "CH1&&CH2", // trig at ch1 or ch2
                                              "CH1+CH2"  // trig at ch1 plus ch2
                                        };

char ExpBuffer [MAXPREUNITS] = { 'f',  // femto
                                 'p',  // pico
                                 'n',  // nano
                                 '',  // micro
                                 'm',  // mili
                                 ' ',  // no multiplier
                                 'K',  // kilo
                                 'M',  // Mega
                                 'G',  // Giga
                                 'T'   // Tera
                               };

/*
 * RAM Data pointers ---> initialized in main()
 */
unsigned far* pGlobalRAMDataCH1;   // global data buffer for ch1, 2*numsamples+100
unsigned far* pGlobalRAMDataCH2;   // global data buffer for ch2, 2*numsamples+100
unsigned far* pLocalRAMDataCH1;    // local data buffer for ch1, 2*horzres
unsigned far* pLocalRAMDataCH2;    // local data buffer for ch2, 2*horzres
unsigned far* pOldLocalRAMDataCH1; // buffer for previous data ch1, 2*horzres
unsigned far* pOldLocalRAMDataCH2; // buffer for previous data ch2, 2*horzres

int nScopeBoxYBottom,  // Y position of bottom of scope box
    nNumFrameYPix,     // number of Y pixels in scope frame
    nNumYPixPerDiv,    // number of Y pixels per division
    nScopeBoxXRight,   // X position of righthand side of scope box
    nNumFrameXPix,     // number of X pixels in scope frame
    nNumXPixPerDiv,    // number of X pixels per division
    nNumSamples = 350; // number of samples

BOOL bFirstLIVE     = TRUE;  // flag for first time display of LIVE
BOOL bKillTimer     = FALSE; // flag for killing timer
BOOL bDataRetrieved = FALSE; // flag for indication data retrieved
BOOL bCanMeasure    = TRUE;  // flag for indicating if can be measured

/*
 *  Local function declarations
 */
 double DeriveFloatingPointNumber (const char* pVoltDivBuffer);
 double AdjustDecimal (char ExpChar, char* pVoltDivBuffer);


/*
 * GlobVars.c function definitions...
 *
 * NAME:
 *   double DeriveFloatingPointNumber (const char* pVoltDivBuffer)
 * DESCRIPTION:
 *   this routine derives a floating point number from a string with a
 *   floating point number. Just using atof() (Ascii TO Float) is not
 *   sufficient, since the string can hold atof-not-valid characters
 *   (e.g. "2.35 mV")
 * INPUT PARAMETERS:
 *   pVoltDivBuffer  holds the string to convert
 * OUTPUT PARAMETERS:
 *   return value:   the float value of the input string
 */
double DeriveFloatingPointNumber (const char* pVoltDivBuffer)
{
 int  nBufIndex, nVoltDivBufIndex = 0;

 /*
  * all valid number-characters
  */
 char Buffer [] = {'0','1','2','3','4','5','6','7','8','9','.'};

 BOOL bNumberFound,        // flag to indicate whether a number is found
      bCharFound = FALSE;  // flag to indicate whether a character is found
 char VoltDivBuffer[10];   // local buffer for the input string
 char ExpChar;             // char to hold an exponent multiplier (p,n,,m,k,...

/*
 * first make a copy of the input string into the local buffer
 */
 strcpy (VoltDivBuffer, pVoltDivBuffer);

/*
 * Now extract Number from pVoltDivBuffer
 */
 do
 {
  /*
   * check for each character in the buffer whether it is a valid character
   * for a number.
   */
   bNumberFound = FALSE;
   nBufIndex = 0;
   do
   {
    /*
     * check whether the character at this position in the buffer is a
     * valid character for a number
     */
    if (VoltDivBuffer[nVoltDivBufIndex] == Buffer[nBufIndex])
    {
     /*
      * Yes it is
      */
      bNumberFound = TRUE;
    }
    else
    {
     /*
      * No it is not, compare to the next valid number character
      */
      nBufIndex++;
    }
  }
  while ((!bNumberFound) && (nBufIndex < 11));
 /*
  * character is either a valid number character or not not found in
  * reference buffer
  */
  if (bNumberFound)
  /*
   * it was a valid number
   */
   nVoltDivBufIndex++;
  else
  /*
   * appeared to be a normal character
   */
   bCharFound = TRUE;
 }
 while (!bCharFound);

/*
 * All valid number characters in the string are found.
 * The next character is probably an exponent multiplier (like: n(ano) k(ilo))
 */
 ExpChar = VoltDivBuffer[nVoltDivBufIndex];

/*
 * terminate the string just at the exponent multiplier (which will be
 * removed from the string.
 */
 VoltDivBuffer [nVoltDivBufIndex] = NULLTERM;

/*
 * Now a string with only a number and an exponent multiplier are available
 * (e.g. "2.35" and "m")
 * convert these to a number using AdjustDecimal() and quit the routine
 */
 return (AdjustDecimal (ExpChar, VoltDivBuffer));

} //DeriveFloatingPointNumber



/* NAME:
 *   double AdjustDecimal (char ExpChar, char* pVoltDivBuffer)
 * DESCRIPTION:
 *   this routine converts a number string and a decimal multiplier to a
 *   floating point number.
 * INPUT PARAMETERS:
 *   pVoltDivBuffer  holds the number string to convert
 *   ExpChar         holds the decimal multiplier
 * OUTPUT PARAMETERS:
 *   return value:   the value of the input string and the decimal multiplier
 */
double AdjustDecimal (char ExpChar, char* pVoltDivBuffer)
{
 double dAdjustedNum;

/*
 * convert the string to a float and apply the decimal multiplier
 */
 switch (ExpChar)
 {
  case 'f':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e-15;
       break;
  case 'p':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e-12;
       break;
  case 'n':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e-9;
       break;
  case '':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e-6;
       break;
  case 'm':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e-3;
       break;
  case 'K':
  case 'k':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e3;
       break;
  case 'M':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e6;
       break;
  case 'G':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e9;
       break;
  case 'T':
       dAdjustedNum = atof (pVoltDivBuffer) * 1e12;
       break;
 /*
  * The next characters are not valid decimal multipliers, but can be found
  * at this position if no decimal multiplier was used in the original
  * string.
  */
  case 'V': // Volts
  case '%':
  case '*':
  case 's': // Sec
  case ' ':
  case 'H': // Hz
       dAdjustedNum = atof (pVoltDivBuffer);
       break;
  default:
       dAdjustedNum = atof (pVoltDivBuffer); // Not adjusted
       break;
 }
 return (dAdjustedNum);
} //AdjustDecimal
