{-------------------------------- TIMER --------------------------------------}
procedure TimeOutTimer;
interrupt;
begin
  inc(TimerCount);
end;{TimeOutTimer}

{*****************************************************************************}
{************************    ProgramExit *************************************}
{*****************************************************************************}
procedure ProgramExit;
begin
  exitproc:=exitsave;
  setintvec($1c,Int1cSave);
  port[Tp112AddressMaster+4]:=3;
  port[Tp112AddressMaster+5]:=0;
  port[Tp112AddressSlave1+4]:=3;
  port[Tp112AddressSlave1+5]:=0;
end;{ProgramExit}

function nClip( nLoLim, nValue, nHiLim : integer ) : integer;
(*
 * DESCRIPTION
 *   This function clips nValue to nLoLim and nHiLim
 * INPUT
 *   nValue      : the value to clip
 *   nLoLim      : the lower limit to clip to
 *   nHiLim      : the higher limit to clip to
 * OUTPUT
 *   returnvalue : the clipped value
 *)
begin

  if   nValue >  nHiLim
  then nValue := nHiLim;

  if   nValue <  nLoLim
  then nValue := nLoLim;

  nClip := nValue;

end; { nClip }



{*****************************************************************************}
{******************** SetPga ************************************************}
{*****************************************************************************}
procedure SetPga( Ch : Integer; AcDc1 : boolean; Value1 : integer; IoAddress : integer);
begin
  value1 := nClip( 0, value1, 4 );
  if not AcDc1 then value1 := value1 or 8;
  PgaRegAd[ch] := value1;
  port[IoAddress+ContReg[ch+1]] := PgaConAd[ch];
  port[IoAddress+DataReg[ch+1]] := PgaRegAd[ch];

end;{SetPga}


{*****************************************************************************}
{******************** ReadEoc ************************************************}
{*****************************************************************************}
function ReadEoc(IoAddress:integer):boolean;
var Tp112Address : word;
begin
  case IoAddress of
    MasterCard : Tp112Address := Tp112AddressMaster;
    SlaveCard1 : Tp112Address := Tp112AddressSlave1;
    else         exit;
  end; { case }
  ReadEoc := (port[Tp112Address+2] and 1) = 1;
end;{ReadEoc}


{*****************************************************************************}
{******************** SetPostCount  ******************************************}
{*****************************************************************************}
procedure SetPostCount(PostCount:word;IoAddress:integer);
var PostLoReg    : byte;
    PostHiReg    : byte;
    Tp112Address : word;
begin
  Case IoAddress of
    MasterCard : Tp112Address := Tp112AddressMaster;
    SlaveCard1 : Tp112Address := Tp112AddressSlave1;
    else       exit;
  end; { case }
{  PostCount                     := PostCount;}
  PostLoReg                     := lo(PostCount);
  PostHiReg                     := hi(PostCount);
  port[Tp112Address+ContReg[1]] := PostLoCon;
  port[Tp112Address+DataReg[1]] := PostLoReg;
  port[Tp112Address+ContReg[1]] := PostHiCon;
  port[Tp112Address+DataReg[1]] := PostHiReg;
end;{SetPostCounter}


{*****************************************************************************}
{******************** SetTrigLevel *******************************************}
{*****************************************************************************}
procedure SetTriglevel(hel:boolean;level,hys,source:byte;IoAddress:integer);
var ChNumber     : byte;
    Help         : integer;
    Tp112Address : word;
begin
  Case IoAddress of
    MasterCard : Tp112Address:=Tp112AddressMaster;
    SlaveCard1 : Tp112Address:=Tp112AddressSlave1;
    else       exit;
  end; { case }
  for ChNumber := 1 to 2 do
  begin
    if hel then
    begin
     (*
      * triggering on positive going slope
      *)
      Level1RegAd[ChNumber] := nClip( 0, level-hys, 255 );
      Level2RegAd[ChNumber] := nClip( 0, level    , 255 );
      if source=ExtTrig then
      begin
        TrigLevelRegAd[ChNumber] := 0;
      end { if }
      else
      begin
        TrigLevelRegAd[ChNumber] := 5;
      end; { else }
    end { if }
    else
    begin
     (*
      * triggering on negative going slope
      *)
      Level1RegAd[ChNumber] := nClip( 0, level+hys, 255 );
      Level2RegAd[ChNumber] := nClip( 0, level    , 255 );
      if source = ExtTrig then
      begin
        TrigLevelRegAd[ChNumber] := 2;
      end { if }
      else
      begin
        TrigLevelRegAd[ChNumber] := 6;
      end; { else }
    end; { else }
    port[Tp112Address+ContReg[ChNumber+1]] := TrigLevelConAd[ChNumber];
    port[Tp112Address+DataReg[ChNumber+1]] := TrigLevelRegAd[ChNumber];

    port[Tp112Address+ContReg[ChNumber+1]] := Level1ConAd[ChNumber];
    port[Tp112Address+DataReg[ChNumber+1]] := Level1RegAd[ChNumber];
    port[Tp112Address+ContReg[ChNumber+1]] := Level2ConAd[ChNumber];
    port[Tp112Address+DataReg[ChNumber+1]] := Level2RegAd[ChNumber];
  end; { for }
end;{SetTriggerLevel}


{*****************************************************************************}
{******************** SetTrigBit    ******************************************}
{*****************************************************************************}
procedure SetTrigbit(IoAddress:integer);
var Tp112Address : word;
begin
  case IoAddress of
    MasterCard : Tp112Address := Tp112AddressMaster;
    SlaveCard1 : Tp112Address := Tp112AddressSlave1;
    else       exit;
  end;
  ModeReg := $80 + $0F;
  port[Tp112Address+0] := ModeCon;
  port[Tp112Address+1] := ModeReg;
end;{SetTriggerBit}


{*****************************************************************************}
{******************** SetTimeBase ********************************************}
{*****************************************************************************}
procedure SetTimebase(var rFreq : real; IoAddress : integer);
const CristalFreq     =  4e+6; { the cristal frequency on the TP112         }
      MaxFreq         =  1e+6; { maximum sampling frequency                 }
      FreqDiv100      =  4e+4; { cristal frequency / 100                    }
      FreqLoLim       =    62; { lower limit frequency to be generated with }
                               { a 16-bits divider (4e6/65535)              }
      Div100          =   100; { division factor for extra divider          }
var   wDivisionFactor : word;
      rInputFreq      : real;
      byTimebaseReg   : byte;
      Tp112Address    : word;

begin
  Case IoAddress of
    MasterCard : Tp112Address := Tp112AddressMaster;
    SlaveCard1 : Tp112Address := Tp112AddressSlave1;
    else       exit;
  end; { case }
 (*
  * the frequencies are generated by a timer chip
  *)

  rInputFreq := CristalFreq;

  if   rFreq >  MaxFreq
  then rFreq := MaxFreq;

  if   rFreq < FreqLoLim
  then rInputFreq := FreqDiv100;

  wDivisionFactor := round( rInputFreq / rFreq );
  rFreq           := rInputFreq / wDivisionFactor;

 (*
  * init the timer IC 8254, control word A1,A0 =11
  *)
  byTimebaseReg                  := 3;
  port[Tp112Address+ContReg[1]] := TimebaseCon;
  port[Tp112Address+DataReg[1]] := byTimebaseReg;
  port[Tp112Address+ContReg[1]] := WriteTimebaseCon;
  port[Tp112Address+DataReg[1]] := 54;  { 00 11 011 0 }
      (*
       * 00  [ = select counter 0                   ]
       * 11  [ = write least significant byte first ]
       * 011 [ = mode 3, divide by n counter        ]
       * 0   [ = 16-bits counter                    ]
       *)
  byTimebaseReg                  := 0;
  port[Tp112Address+ContReg[1]] := TimebaseCon;
  port[Tp112Address+DataReg[1]] := TimebaseReg;
  port[Tp112Address+ContReg[1]] := WriteTimebaseCon;
  port[Tp112Address+DataReg[1]] := lo(wDivisionFactor);
  port[Tp112Address+DataReg[1]] := hi(wDivisionFactor);

  if   rFreq < FreqLoLim
  then begin
        (*
         * extra divider is needed
         *)
         byTimebaseReg                  := 3;
         port[Tp112Address+ContReg[1]] := TimebaseCon;
         port[Tp112Address+DataReg[1]] := byTimebaseReg;
         port[Tp112Address+ContReg[1]] := WriteTimebaseCon;
         port[Tp112Address+DataReg[1]] := 118;  { 01 11 011 0 }
             (*
              * 01  [ =select counter 1                   ]
              * 11  [ =write least significant byte first ]
              * 011 [ =mode 3, divide by n counter        ]
              * 0   [ =16-bits counter                    ]
              *)
         byTimebaseReg                  := 1;
         port[Tp112Address+ContReg[1]] := TimebaseCon;
         port[Tp112Address+DataReg[1]] := byTimebaseReg;
         port[Tp112Address+ContReg[1]] := WriteTimebaseCon;
         port[Tp112Address+DataReg[1]] := lo(Div100);
         port[Tp112Address+DataReg[1]] := hi(Div100);

        (*
         * select clock input b
         *)
         byTimebaseReg := 8;
       end
  else begin
        (*
         * can be made with one divider, so no extra divider needed
         * select clock input a
         *)
         byTimebaseReg := 0;
       end;

  port[Tp112Address+ContReg[1]] := TimebaseCon;
  port[Tp112Address+DataReg[1]] := TimebaseReg;

end;{SetTimeBase}




{*****************************************************************************}
{******************** ClearSram **********************************************}
{*****************************************************************************}
procedure ClearSram(IoAddress:integer);
var ch_nr       : integer;
    Freq        : real;
    Tp112Address: word;
begin
  Case IoAddress of
    MasterCard : Tp112Address:=Tp112AddressMaster;
    SlaveCard1 : Tp112Address:=Tp112AddressSlave1;
    else       exit;
  end;
  {reset trig channel 1}
  port[Tp112Address+ContReg[2]] := TrigResetConAd[1];
  port[Tp112Address+DataReg[2]] := 0;
  {reset trig channel 2}
  port[Tp112Address+ContReg[3]] := TrigResetConAd[2];
  port[Tp112Address+DataReg[3]] := 0;
  {set clck source to PC}
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := 0;
  SetPostCount(32768+1,IoAddress);
  for ch_nr:=1 to 2 do
  begin
    {set data/zero on zero}
    port[Tp112Address+ContReg[ch_nr+1]] := DataZeroConAd[ch_nr];
    port[Tp112Address+DataReg[ch_nr+1]] := 1;
  end;
  {set fastest timebase}
  Freq:=1e6;
  SetTimebase(Freq,MasterCard);
  {set clck source}
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := 14;
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {set clck on}
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := 15;
end; {ClearSram}


{*****************************************************************************}
{******************** GotoSramPos ********************************************}
{*****************************************************************************}
procedure GotoSramPos(Pos,IoAddress:integer);
var Freq         : real;
    Tp112Address : word;

begin
  Case IoAddress of
    MasterCard : Tp112Address := Tp112AddressMaster;
    SlaveCard1 : Tp112Address := Tp112AddressSlave1;
    else       exit;
  end;
  {set clck source to PC}
  ModeReg := 0;
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg;
  SetPostCount(Pos,IoAddress);
  Freq:=1e6;
  SetTimebase(Freq,MasterCard);
  {reset post}
  port[Tp112Address+0] := ResetCon;
  port[Tp112Address+1] := 0;
  {set clck source}
  ModeReg := 129;
  port[Tp112Address+0] := ModeCon;
  port[Tp112Address+1] := ModeReg;
  {set clck on}
  ModeReg := 131;
  port[Tp112Address+0] := ModeCon;
  port[Tp112Address+1] := ModeReg;

  repeat
    { empty }
  until (port[Tp112Address+2] and 1)=1;

  ModeReg := 0;
  port[Tp112Address+0] := ModeCon;
  port[Tp112Address+1] := ModeReg;
end; {GotoSramPos}

{*****************************************************************************}
{******************** ReadAddressCounter *************************************}
{*****************************************************************************}
function ReadAddressCounter(IoAddress:integer):word;
var Count        : word;
    Tp112Address : word;
begin
  Case IoAddress of
    MasterCard : Tp112Address:=Tp112AddressMaster;
    SlaveCard1 : Tp112Address:=Tp112AddressSlave1;
    else       exit;
  end;
  {set clck source to PC}
  ModeReg := 0;
  port[Tp112Address+ContReg[1]]:= ModeCon;
  port[Tp112Address+DataReg[1]]:= ModeReg;
  {read address counter}
  Count:=port[Tp112Address+0];
  Count:=port[Tp112Address+1]*256+Count;
  ReadAddressCounter:=Count and 32767;
end;{ReadAddressCounter}

{*****************************************************************************}
{******************** ReadDataSram *******************************************}
{*****************************************************************************}
procedure ReadDataSram(instel : Tp112SettingsType; var Data: MeasureDataArray);
var i             : word;
    SampleNumber  : integer;
    Help_i        : longint;
    CountSlave1,
    Count         : word;
    LoByte,
    HiByte : byte;
begin
  CountSlave1:=ReadAddressCounter(SlaveCard1);

  Count:=ReadAddressCounter(MasterCard);
  Help_i:=Count-instel.PostSamples-instel.PreSamples;
  if Help_i<0 then Help_i:=Help_i+32767;
  GotoSramPos(Help_i,MasterCard);
  for SampleNumber:=-instel.PreSamples to instel.PostSamples-1 do
  begin
    {Read Data}
    LoByte := port[Tp112AddressMaster+4];
    HiByte := port[Tp112AddressMaster+5] and $0F;
    Data[MasterCard][1][SampleNumber] := 256 * HiByte + LoByte;

    LoByte := port[Tp112AddressMaster+6];
    HiByte := port[Tp112AddressMaster+7] and $0F;
    Data[MasterCard][2][SampleNumber] := 256 * HiByte + LoByte;
    {inc address Counter}
    port[Tp112AddressMaster+ContReg[1]] := CountCon;
    port[Tp112AddressMaster+DataReg[1]] := 0;
  end;
  ClearSram(MasterCard);


  Help_i:=CountSlave1-instel.PostSamples-instel.PreSamples;
  if Help_i<0 then Help_i:=Help_i+32767;
  GotoSramPos(Help_i,SlaveCard1);
  for SampleNumber:=-instel.PreSamples to instel.PostSamples-1 do
  begin
    {Read Data}
    LoByte := port[Tp112AddressSlave1+4];
    HiByte := port[Tp112AddressSlave1+5] and $0F;
    Data[SlaveCard1][1][SampleNumber] :=  256 * HiByte + LoByte;

    LoByte := port[Tp112AddressSlave1+6];
    HiByte := port[Tp112AddressSlave1+7] and $0F;
    Data[SlaveCard1][2][SampleNumber] :=  256 * HiByte + LoByte;

    {inc address Counter}
    port[Tp112AddressSlave1+ContReg[1]] := CountCon;
    port[Tp112AddressSlave1+DataReg[1]] := 0;
  end;
  ClearSram(SlaveCard1);
end;{ReadDataSram}


{*****************************************************************************}
{******************** MeasureTp112 ******************************************}
{*****************************************************************************}
procedure MeasureTp112(instel : Tp112SettingsType);
Var Tp112Address : word;
    ModeReg1,
    ModeReg2     : byte;
begin
  {set clck source}
  Tp112Address:=Tp112AddressMaster;
  ModeReg1 := 0;
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg1;

  SetTriglevel(instel.TrigHel,instel.TrigLevel,instel.TrigHys,instel.TrigSource,MasterCard);
  SetTimebase (instel.SampleFreq, MasterCard);
  SetPostCount(instel.PostSamples,MasterCard);
  SetPostCount(instel.PostSamples,SlaveCard1);
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {reset trig channel 1}
  port[Tp112Address+ContReg[2]] := TrigResetConAd[1];
  port[Tp112Address+DataReg[2]] := 0;
  {reset trig channel 2}
  port[Tp112Address+ContReg[3]] := TrigResetConAd[2];
  port[Tp112Address+DataReg[3]] := 0;

  Tp112Address:=Tp112AddressSlave1;
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {set clck source}
  ModeReg2 := 0;
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg2;
 (*
  * the following instruction is crucial: set the trigger mode of the slave
  * card to external
  *)
  ModeReg2 := ModeReg or (128 + 64);
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg2;
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {reset trig channel 1}
  port[Tp112Address+ContReg[2]] := TrigResetConAd[1];
  port[Tp112Address+DataReg[2]] := 0;
  {reset trig channel 2}
  port[Tp112Address+ContReg[3]] := TrigResetConAd[2];
  port[Tp112Address+DataReg[3]] := 0;
  { Pass throug AD data }
  port[Tp112Address+ContReg[2]]:=4;
  port[Tp112Address+DataReg[2]]:=0;
  { Pass throug AD data }
  port[Tp112Address+ContReg[3]]:=4;
  port[Tp112Address+DataReg[3]]:=0;

  Tp112Address := Tp112AddressMaster;
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {set clck source}
  ModeReg1 := 0;
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg1;
  {set trigger source}

  case instel.TrigSource of
    Ch1Trig : ModeReg1 :=   0;
    Ch2Trig : ModeReg1 :=  64;
    NoTrig  : ModeReg1 := 128;
    ExtTrig : ModeReg1 := 192;
    else      ModeReg1 := 128;
  end;{case}
  port[Tp112Address+ContReg[1]] := ModeCon;
  port[Tp112Address+DataReg[1]] := ModeReg1;
  {reset post counter and address counter}
  port[Tp112Address+ContReg[1]] := ResetCon;
  port[Tp112Address+DataReg[1]] := 0;
  {reset trig channel 1}
  port[Tp112Address+ContReg[2]] := TrigResetConAd[1];
  port[Tp112Address+DataReg[2]] := 0;
  {reset trig channel 2}
  port[Tp112Address+ContReg[3]] := TrigResetConAd[2];
  port[Tp112Address+DataReg[3]] := 0;
  {Pass throug AD data channel 1}
  port[Tp112Address+ContReg[2]]:=4;
  port[Tp112Address+DataReg[2]]:=0;
  {Pass throug AD data channel 2}
  port[Tp112Address+ContReg[3]]:=4;
  port[Tp112Address+DataReg[3]]:=0;

  {set clck sram}
  port[Tp112AddressMaster+ContReg[1]] := ModeCon;
  port[Tp112AddressSlave1+ContReg[1]] := ModeCon;
  port[Tp112AddressSlave1+DataReg[1]] := ModeReg2 or $0F;
  port[Tp112AddressMaster+DataReg[1]] := ModeReg1 or $0F;
end;{MeasureTp112}


{*****************************************************************************}
{************************       HardwarePresent    ***************************}
{*****************************************************************************}
function HardwarePresent:boolean;
var i     : integer;
    Value : integer;
begin
  Value:=0;
  for i:=1 to 10 do Value:=port[Tp112AddressMaster+3]+Value;
  for i:=1 to 10 do Value:=port[Tp112AddressSlave1+3]+Value;
  if Value=0 then HardwarePresent:=true
             else HardwarePresent:=false;
end;


{*****************************************************************************}
{************************       InitTp112          ***************************}
{*****************************************************************************}
procedure InitTp112;
var ChNumber,
    Switch   : integer;
    CalError : boolean;
begin
  for i:=-MaxPreSamples to MaxPostSamples do
  begin
    MeasureData[1][1][i] := 2048;
    MeasureData[1][2][i] := 2048;
    MeasureData[2][1][i] := 2048;
    MeasureData[2][2][i] := 2048;
    OldData    [1][1][i] := 2048;
    OldData    [1][2][i] := 2048;
    OldData    [2][1][i] := 2048;
    OldData    [2][2][i] := 2048;
  end;

  getintvec($1c,Int1cSave);
  setintvec($1c,@TimeOutTimer);
  if not(HardwarePresent) then
  begin
    write(Text2);
    halt(1);
  end;

  PgaSwitch[1][ 0] := 4;
  PgaSwitch[1][ 1] := 3;
  PgaSwitch[1][ 2] := 2;
  PgaSwitch[1][ 3] := 1;
  PgaSwitch[1][ 4] := 0;

  PgaSwitch[2][ 0] := 4;
  PgaSwitch[2][ 1] := 3;
  PgaSwitch[2][ 2] := 2;
  PgaSwitch[2][ 3] := 1;
  PgaSwitch[2][ 4] := 0;

  PgaSwitchValue[0] :=  1.25;
  PgaSwitchValue[1] :=  2.5;
  PgaSwitchValue[2] :=  5;
  PgaSwitchValue[3] := 10;
  PgaSwitchValue[4] := 20;

  ClearSram(MasterCard);
  ClearSram(SlaveCard1);
end;


{*****************************************************************************}
{************************    MeasureTp112          ***************************}
{*****************************************************************************}
procedure StartMeasureTp112(MeasureSetting :MeasureSettingsArray;var MeasureData:MeasureDataArray);
var Tp112Setting    : Tp112SettingsType;
    SwitchSetting   : longint;
begin
  SetPga(Ch1, MeasureSetting[MasterCard].AcDc[Ch1], PgaSwitch[1][MeasureSetting[MasterCard].Switch[1]], TP112AddressMaster);
  SetPga(Ch2, MeasureSetting[MasterCard].AcDc[Ch2], PgaSwitch[2][MeasureSetting[MasterCard].Switch[2]], TP112AddressMaster);


  SetPga(Ch1, MeasureSetting[SlaveCard1].AcDc[Ch1], PgaSwitch[1][MeasureSetting[SlaveCard1].Switch[1]], TP112AddressSlave1);
  SetPga(Ch2, MeasureSetting[SlaveCard1].AcDc[Ch2], PgaSwitch[2][MeasureSetting[SlaveCard1].Switch[2]], TP112AddressSlave1);

  Tp112Setting.SampleFreq := MeasureSetting[MasterCard].SampleFreq;
  Tp112Setting.PostSamples:= MeasureSetting[MasterCard].PostSamples;
  Tp112Setting.PreSamples := MeasureSetting[MasterCard].PreSamples;
  Tp112Setting.TrigHys    := MeasureSetting[MasterCard].TrigHys;
  Tp112Setting.TrigLevel  := MeasureSetting[MasterCard].TrigLevel;
  Tp112Setting.TrigHel    := MeasureSetting[MasterCard].TrigHel;
  Tp112Setting.TrigSource := MeasureSetting[MasterCard].TrigSource;
  MeasureTp112(Tp112Setting);


  asm
    CLI             { disable interrupts    }
  end;
  TimerCount := 0;  { clear timeout counter }
  asm
    STI             { enable interrupts     }
  end;

  repeat
    if (TimerCount>=MeasureSetting[MasterCard].TimeOut) then
    begin
      SetTrigbit(MasterCard);
    end; { if }
  until ReadEoc(MasterCard);

  if ReadEoc(MasterCard) then
  begin
    ReadDataSram(Tp112Setting,MeasureData);
  end; { if }
end; { StartMeasureTp112 }
