(*
 * This file contains HS508 specific routines, consts etc.
 *)


const

      LPT1Address      = $378;
      LPT2Address      = $278;
      LPT3Address      = $3BC;
      TP208Address     = $300;
      TP508Address     = $300;
      TP112Address     = $300;

      PPControl        = 4; { 0100 }

      PPDevSel         = 1; { 0001 } { STROBE    }
      PPAdrSel         = 2; { 0010 } { AUTO_FDXT }
      PPIOW            = 4; { 0100 } { INIT      }
      PPIOR            = 8; { 1000 } { SLCT_IN   }

      PiBiReg          = 0;
      PiExtClckReg     = 1;
      PiRelaisReg      = 2;
      PiTimerReg       = 3;
      PiEEpromReg      = 4;
      PiOutReg         = 5;

     (*
      * Address constants within the HS508 EEprom
      *)
      EEpromTypeId       =   1; {identify of the device at the Parallel Port  }
      EEpromSerialNr1    =   5; {type indication:1 byte, serial number:3bytes }
      EEpromCalDateTime  =  10; {year:2 bytes,month:1 byte,day:1 byte,hour:1 byte,min:1 byte}
      EEpromAdrGainCh1   =  50; {12 gain settings, each is two bytes, total 24 bytes}
      EEpromAdrGainCh2   =  75;
      EEpromAdrGainCh3   = 100;
      EEpromAdrOffsetCh1 = 125;
      EEpromAdrOffsetCh2 = 150;
      EEpromAdrOffsetCh3 = 175;
      EEpromSerialNr2    = 200;

      HS508DeviceNrExt   = 0;
      HS508DeviceNrInt   = 0+16;

      IDdevice           = 170; { this code is placed in each EEprom }
      HSDeviceTypeID     =   2; { device ID for HS508, written in EEPROM }

      ContReg          : array[1..3] of Byte = ( 0, 4, 6 );
      DataReg          : array[1..3] of Byte = ( 1, 5, 7 );

      PostLoCon        = 0;
      PostHiCon        = 1;
      TimebaseCon      = 2;
      ModeCon          = 3;
      ResetCon         = 4;
      CountCon         = 5;
      WriteTimebaseCon = 6;

      Level1ConAd      : array[1..2] of Byte = ( 0, 0 );
      Level2ConAd      : array[1..2] of Byte = ( 1, 1 );
      TrigLevelConAd   : array[1..2] of Byte = ( 2, 2 );
      PGAConAd         : array[1..2] of Byte = ( 3, 3 );
      DateZeroConAd    : array[1..2] of Byte = ( 4, 4 );
      IRQConAd         : array[1..2] of Byte = ( 5, 5 );
      TrigResetConAd   : array[1..2] of Byte = ( 6, 6 );

      Level1RegAd      : array[1..2] of Byte = ( 0, 0 );
      Level2RegAd      : array[1..2] of Byte = ( 0, 0 );
      TrigLevRegAd     : array[1..2] of Byte = ( 0, 0 );
      PgaRegAd         : array[1..2] of Byte = ( 0, 0 );
      TrigResetRegAd   : array[1..2] of Byte = ( 0, 0 );

      PostLoReg        : byte = 0;
      PostHiReg        : byte = 0;
      OutputReg        : byte = 0;
      TimebaseReg      : byte = 0;
      ModeReg          : byte = 0;
      ResetReg         : byte = 0;
      ContrReg         : byte = 0;
      WriteTimebaseReg : byte = 0;

      PGASwitchTime    = 10;

const Pga20mVolt  =  1;
      Pga40mVolt  =  2;
      Pga80mVolt  =  3;
      Pga200mVolt =  4;
      Pga400mVolt =  5;
      Pga800mVolt =  6;
      Pga2Volt    =  7;
      Pga4Volt    =  8;
      Pga8Volt    =  9;
      Pga20Volt   = 10;
      Pga40Volt   = 11;
      Pga80Volt   = 12;

      Pga2Att1    =      1;  { 00 0000 0001 }
      Pga2Att10   =      2;  { 00 0000 0010 }
      Pga2Att100  =      4;  { 00 0000 0100 }
      Pga2AcDc    =      8;  { 00 0000 1000 }
      Pga1AcDc    =     16;  { 00 0001 0000 }
      Pga1Att1    =     32;  { 00 0010 0000 }
      Pga1Att10   =     64;  { 00 0100 0000 }
      Pga1Att100  =    128;  { 00 1000 0000 }
      Pga1on      =    256;  { 01 0000 0000 }
      Pga2on      =    512;  { 10 0000 0000 }

      MaxSetting  = 12;      { Hightest input range setting available }
      MinSetting  =  4;      { Lowest   input range setting available }

      CristalFreq     = 25e+6;
      MaxSamplingFreq = 25e+6;

      DeviceMaxValue  = 255;       { 8 bits instrument }

      DeviceString = 'HS508';

      DcOffsetGain = 0.57;


function RemoveSpaces( str : string ) : string;
(*
 * DESCRIPTION
 *   This routine removes leading spaces from a string
 * INPUT
 *   str         : the input string
 * OUTPUT
 *   returnvalue : the 'cleaned' string
 *)
begin

  while ( str[1] = ' ' ) do
    delete(str,1,1);
  RemoveSpaces := str;

end; { RemoveSpaces }


function Int2Str(  lValue     : longint;
                   nWidth     : integer;
                   bRemSpaces : boolean
                ) : string;
(*
 * DESCRIPTION
 *   This routine converts a number to a string, where optionally leading
 *   spaces can be removed.
 * INPUT
 *   lValue      : the value to convert to a string
 *   nWidth      : the number of characters the string has to use
 *   bRemSpaces  : flag to indicate whether spaces have to be removed
 * OUTPUT
 *   returnvalue : str string which is made from the value
 *)
var st : string;
begin

  str( lValue:nWidth, st );
  if   bRemSpaces
  then int2str := RemoveSpaces( st )
  else int2str := st;

end; { Int2Sstr }



procedure TimeOutTimer;
(*
 * DESCRIPTION
 *   This interrupt routine is called 18.2 times per second. It is used to
 *   determine whether the trigger system is timed out and a start of
 *   measurement is to be forced.
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
interrupt;
begin

  inc(lTimerCount );

end; { TimeOutTimer }

procedure ProgramExit;
(*
 * DESCRIPTION
 *   This routine is called when the program is to be terminated.
 *   It brings the computer back to its normal state.
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin

 (*
  * restore original exit procedure
  *)
  exitproc := exitsave;

 (*
  * restore original timer interrupt routine
  *)
  SetIntVec( $1C, int1Csave );

end; { ProgramExit }


procedure SelectDeviceNrPI( byDeviceNr : Byte );
(*
 * DESCRIPTION
 *   This routine selects either a P(arallel) P(rinter) I(nterface)
 *   internal or PPI external register.
 *   This has to be done before any other accessing is done to the HS508
 * INPUT
 *   byDeviceNr:  HS508DeviceNrExt  a register outside the PPI
 *                HS508DeviceNrInt  a register inside the PPI
 * OUTPUT
 *   -
 *)
var DeviceAddress_2 : integer;
begin
  DeviceAddress_2 := DeviceAddress + 2;
(*
 * port[PiPortAddres+2]:=PPControl;
 * port[PiPortAddres  ]:=DeviceNr;
 * port[PiPortAddres+2]:=PPControl or       PPDevSel;
 * port[PiPortAddres+2]:=PPControl and (255-PPDevSel);
 *)
  asm
    MOV DX,DeviceAddress_2
    MOV AL,PPControl
    OUT DX,AL

    MOV DX,DeviceAddress
    MOV AL,byDeviceNr
    OUT DX,AL
    MOV DX,DeviceAddress_2

    MOV AL,(PPDevSel or PPControl)
    OUT DX,AL
    MOV AL,PPControl AND (255-PPDevSel)
    OUT DX,AL
  end;

end;{SelectDeviceNrPI}


procedure LoadDataToPI( byAddress, byData : byte );
(*
 * DESCRIPTION
 *   This routine loads a byte to the PPI, to the specified address.
 *   Before this is done, there has to be selected whether it should
 *   be a PPI internal or PPI external address
 * INPUT
 *   byAddress :  the address to load the data to
 *   byData    :  the data to load
 * OUTPUT
 *   -
 *)
var DeviceAddress_2 : integer;
begin
  DeviceAddress_2 := DeviceAddress + 2;
 (*
  * port[DeviceAddress+2] := PPControl;
  * port[DeviceAddress+0] := Adr;
  * port[DeviceAddress+2] := PPControl or       PPAdrSel;
  * port[DeviceAddress+2] := PPControl and (255-PPAdrSel);
  * port[DeviceAddress  ] := Data;
  * port[DeviceAddress+2] := PPControl and       PPIOW;
  * port[DeviceAddress+2] := PPControl or  (255-PPIOW);
  *)

  asm
    MOV DX,DeviceAddress_2
    MOV AL,PPControl
    OUT DX,AL

    MOV DX,DeviceAddress
    MOV AL,byAddress
    OUT DX,AL

    MOV DX,DeviceAddress_2
    MOV AL,(PPAdrSel OR PPControl)
    OUT DX,AL
    AND AL,(255-PPAdrSel)
    OUT DX,AL

    MOV DX,DeviceAddress
    MOV AL,byData
    OUT DX,AL

    MOV DX,DeviceAddress_2
    MOV AL,(255-PPIOW) AND PPControl
    OUT DX,AL
    MOV AL,PPIOW OR PPControl
    OUT DX,AL
  end;
end;{LoadDataToPI}


function ReadDataFromPI( byAdr : byte ) : integer;
(*
 * DESCRIPTION
 *   This routine reads a byte from the PPI, from the specified address.
 *   Before this is done, there has to be selected whether it should
 *   be a PPI internal or PPI external address
 * INPUT
 *   byAdr       : the address to load the data from
 * OUTPUT
 *   returnvalue : the data that was read
 *)
var nLoData,
    nHiData : integer;
begin

  port[DeviceAddress  ] := byAdr;
  port[DeviceAddress+2] := PPControl or PPAdrSel;
  port[DeviceAddress+2] := PPControl;

  port[DeviceAddress+2] := PPControl or PPIOR;
  port[DeviceAddress+2] := PPControl;

  port[DeviceAddress+2] := PPControl or PPAdrSel;
  port[DeviceAddress  ] := 1;
  nHiData               := (((port[DeviceAddress+1] xor 128) shl 1) and 240);
  port[DeviceAddress  ] := 0;
  nLoData               := (((port[DeviceAddress+1] xor 128) shr 3) and 15);
  ReadDataFromPI        := nHiData + nLoData;
  port[DeviceAddress+2] := PPControl;

end;{ReadDataFromPI}



procedure SetRef( nValue1, nValue2 : integer );
(*
 * DESCRIPTION
 *   This routine is not available for the HS508, but present for uniformity.
 *   Therefor it is not filled with instructions.
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin
end;


procedure SetPga ( bCouplingCh1AC, bCouplingCh2AC : boolean; lValue : longint );
(*
 * DESCRIPTION
 *   This routine sets the coupling switches for Ch1 and Ch2 and the
 *   attenuators for both Ch1 and Ch2.
 *   The data is sent serial
 * INPUT
 *   bCouplingCh1AC     : the requested state for the Ch1 coupling switch
 *   bCouplingCh2AC     : the requested state for the Ch2 coupling switch
 *   lValue             : a combined parameter for the attenuator switches
 * OUTPUT
 *   -
 *)
const PgaBlnk         = 64;
      PgaSlct         = 16;
      SerialDataLength =  9;  { 10 bits to send  }
var   bySclck : byte;         { Serial clock bit }
      bySdata : byte;         { Serial data bit  }
      lHelp   : longint;
      Index   : integer;
begin

 (*
  * select the correct device
  *)
  SelectDeviceNrPI( HS508DeviceNrExt );

 (*
  * Add the coupling switch settings to lValue
  *)
  if   not( bCouplingCh1AC )
  then lValue := lValue + PGA1ACDC;
  if   not( bCouplingCh2AC )
  then lValue := lValue + PGA2ACDC;

 (*
  * init the serial communication
  *)
  LoadDataToPI( 4, 3 );
  lHelp := round(exp(SerialDataLength*ln(2)));
  bySclck := 0;
  bySdata := 0;
  LoadDataToPI( 5, (bySdata+bySclck) or PgaBlnk );

 (*
  * send the 10 bits to the switch driver circuitry
  *)
  for Index := 0 to SerialDataLength do
  begin
    if   (lValue and lHelp) = 0
    then bySdata := 0
    else bySdata := 1;
    bySclck := 0;
    LoadDataToPI( 5, (bySdata+bySclck) or PgaBlnk );
    bySclck := 2;
    LoadDataToPI( 5, (bySdata+bySclck) or PgaBlnk );
    lHelp := lHelp div 2;
  end;

 (*
  * Back to normal state
  *)
  LoadDataToPI( 5, PgaSlct or PgaBlnk );
  LoadDataToPI( 5, PgaBlnk );
  delay( PGASwitchtime );

end; {SetPga}


procedure SetDA( nCh : integer; rValue : Real );
(*
 * DESCRIPTION
 *   This procedure sets the DC offset level for a specified channel
 *   the value has to be sent serial
 * INPUT
 *   nCh       : indicate which channel to change
 *   rValue    : the new value for the DAC
 * OUTPUT
 *   -
 *)
const PgaBlnk          = 64;
      AdSlct           =  8;
      SerialDataLength = 15;  { 16 bits to send  }
var   bySclck : byte;         { serial clock bit }
      bySdata : byte;         { serial data bit  }
      lHelp   : longint;
      Index   : integer;
      lData   : longint;
begin
  SelectDeviceNrPI(HS508DeviceNrExt);
 (*
  * Start initialization serial data
  *)
  LoadDataToPI( 4, 3 );

 (*
  * Scale, shift, clip and round the Value
  *)
  rValue := ((rValue/((2.8-0.3)/2))*512)+512;
  if   rValue >  1023
  then rValue := 1023;
  if   rValue <  0
  then rValue := 0;
  lData := round(rValue);
  lData := ($9 shl 12) + (nCh shl 10) + lData;

 (*
  * Continue init serial communication
  *)
  lHelp   := round(exp(SerialDataLength*ln(2)));
  bySclck := 0;
  bySdata := 0;
  LoadDataToPI(5,(bySdata+bySclck+AdSlct) or PgaBlnk);

 (*
  * send all 16 bits
  *)
  for Index := 0 to SerialDataLength do
  begin
    if   (lData and lHelp) = 0
    then bySdata := 0
    else bySdata := 1;
    bySclck := 0;
    LoadDataToPI( 5, (bySdata + bySclck + AdSlct) or PgaBlnk );
    bySclck := 2;
    LoadDataToPI( 5, (bySdata + bySclck + AdSlct) or PgaBlnk );
    lHelp   := lHelp div 2;
  end;

 (*
  * Back to normal state
  *)
  bySclck := 0;
  LoadDataToPI( 5, (bySdata + bySclck + AdSlct) or PgaBlnk );
  bySclck := 2;
  LoadDataToPI( 5, (bySdata + bySclck + AdSlct) or PgaBlnk );
  LoadDataToPI( 5, PgaBlnk );
  LoadDataToPI( 4, 8 );

end; { SetDA }


function EOC : boolean;
(*
 * DESCRIPTION
 *   This routine reads the End Of Converion bit
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue        : the state of the End Of Conversion bit
 *)
begin

  SelectDeviceNrPI( HS508DeviceNrExt );
  EOC := (ReadDataFromPI(2) and 1) = 1; {1111 1001}
  EOC := (ReadDataFromPI(2) and 1) = 1; {1111 1001}
 (*
  * read twice to get delay, needed because of misreadings through some
  * parallel interfaces in PC's. (Problem in the PC, not in the HS508
  * or software
  *)

end; { EOC }


procedure SetPostCount( wPostCount : Word );
(*
 * DESCRIPTION
 *   This routine sets the postcounter to the requested value
 * INPUT
 *   wPostCount : the requested value for the postcounter
 * OUTPUT
 *   -
 *)
var byPostLoReg : byte;
    byPostHiReg : byte;
begin

  SelectDeviceNrPI( HS508DeviceNrExt );
  wPostCount   := wPostCount;
  byPostLoReg  := lo( wPostCount );
  byPostHiReg  := hi( wPostCount );
  LoadDataToPI( ContReg[1], PostLoCon   );
  LoadDataToPI( Datareg[1], byPostLoReg );
  LoadDataToPI( ContReg[1], PostHiCon   );
  LoadDataToPI( Datareg[1], byPostHiReg );

end; { SetPostCount }


procedure SetTrigLevel( InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION
 *   This routine sets the trigger level for both channels
 * INPUT
 *   InstrumentSettings : a structure containing the instrument settings
 * OUTPUT
 *   -
 *)
var nCh   : byte;
    nHelp : integer;

    function nClip( nValue : integer ) : integer;
   (*
    * This local function clips the integer input value nValue between
    * 0 and 255
    *)
    begin
      if   nValue >  255
      then nValue := 255;
      if   nValue <    0
      then nValue :=   0;
      nClip := nValue
    end;  { nClip }

begin

  SelectDeviceNrPI(HS508DeviceNrExt);

 (*
  * do both channels
  *)
  for nCh := Ch1 to Ch2 do
  begin
    with InstrumentSettings do
    begin
     (*
      * The trigger system has two trigger levels, Level1 and Level2.
      * Setup the two levels and the source
      *)
      if   bTrigSlope
      then begin
            (*
             * Level1 has to be 'hysteresis' lower than Level2
             *)
             Level1RegAd[nCh] := nClip( byTrigLevel[nCh] - byTrigHys );
             Level2RegAd[nCh] := nCLip( byTrigLevel[nCh] );
             case byTrigSource of
               ExtTrig : TrigLevRegAd[nCh] := 1+16;  {0000 0001}
               else      TrigLevRegAd[nCh] := 5;     {0000 0101}
             end; { case }
           end  { then }
      else begin
            (*
             * Level1 has to be 'hysteresis' higher than Level2
             *)
             Level1RegAd[nCh] := nClip( byTrigLevel[nCh] + byTrigHys );
             Level2RegAd[nCh] := nClip( byTrigLevel[nCh] );
             case byTrigSource of
               ExtTrig : TrigLevRegAd[nCh] := 0+16;   {0000 0000}
               else      TrigLevRegAd[nCh] := 6;      {0000 0110}
             end; { case }
           end; { else }
    end; { with }

   (*
    * send the values to the trigger circuitry
    *)
    LoadDataToPI(ContReg[nCh+1],TrigLevelConAd[nCh]);
    LoadDataToPI(DataReg[nCh+1],TrigLevRegAd[nCh]);

    LoadDataToPI(ContReg[nCh+1],Level1ConAd[nCh]);
    LoadDataToPI(DataReg[nCh+1],Level1RegAd[nCh]);
    LoadDataToPI(ContReg[nCh+1],Level2ConAd[nCh]);
    LoadDataToPI(DataReg[nCh+1],Level2RegAd[nCh]);
  end;

end; { SetTrigLevel }


procedure SetTrigbit;
(*
 * DESCRIPTION
 *   This routine sets the triggerbit, to force a start of measurement
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin

  SelectDeviceNrPI( HS508DeviceNrExt );
  LoadDataToPI( 0, 7 );
  LoadDataToPI( 1, 2 );

end; { SetTrigBit }


procedure SetTimebase( var rFreq : Real );
(*
 * DESCRIPTION
 *   This routine sets the timebase of the HS508 to the requested sampling
 *   frequency
 * INPUT
 *   rFreq      : the requested sampling frequency
 * OUTPUT
 *   rFreq      : the set frequency ( not all frequencies are available )
 * CAVEAT
 *   This routine has several exit points
 *)
const MaxFreq       = 25.0e+6; { Max sampling frequency HS508               }
      FreqDiv2      = 12.5e+6; { Max sampling frequency / 2                 }
      FreqDiv5      =  5.0e+6; { Max sampling frequency / 5                 }
      FreqDiv10     =  2.5e+6; { Max sampling frequency / 10                }
      FreqDiv500    =  5.0e+4; { Max sampling frequency / 500               }
      FreqLoLim     =      80; { lower limit frequency to be generated with }
                               { one 16-bit divider (5e+6 / 65535)          }
      Div100        =     100; { used for divide by 100                     }
var wDivisionFactor : Word;
    rInputFreq      : Real;
begin

  SelectDeviceNrPI(HS508DeviceNrExt);

 (*
  * The three fastest sampling frequencies are made by dividing the
  * clock oscillator by 1, 2 or 5, in the FPGA
  *)
  if ( rFreq > FreqDiv2 )
  then begin
         TimeBaseReg := 0 + 32;
         LoadDataToPI( ContReg[1], TimeBaseCon );
         LoadDataToPI( DataReg[1], TimeBaseReg );
         exit;
       end;

  if ( rFreq <= FreqDiv2 ) and ( rFreq > FreqDiv5 )
  then begin
         TimeBaseReg := 4 + 32;
         LoadDataToPI( ContReg[1], TimeBaseCon );
         LoadDataToPI( DataReg[1], TimeBaseReg );
         exit;
       end;

  if ( rFreq <= FreqDiv5 ) and ( rFreq > FreqDiv10 )
  then begin
         TimeBaseReg := 8 + 32;
         LoadDataToPI( ContReg[1], TimeBaseCon );
         LoadDataToPI( DataReg[1], TimeBaseReg );
         exit;
       end;

 (*
  * the rest of the sampling frequencies are generated by a timer chip
  *)

  rInputFreq := FreqDiv5;

  if   rFreq >  FreqDiv5
  then rFreq := FreqDiv5;

  if   rFreq < FreqLoLim
  then rInputFreq := FreqDiv500;

  wDivisionFactor := round( rInputFreq / rFreq );
  rFreq           := rInputFreq / wDivisionFactor;


 (*
  * init the timer IC 8254, control word A1,A0=11
  *)
  TimeBaseReg := 3+32;

 (*
  * activate the timebase register
  *)
  LoadDataToPI( ContReg[1], TimeBaseCon );
  LoadDataToPI( DataReg[1], TimeBaseReg );
  LoadDataToPI( ContReg[1], WriteTimebaseCon );
  LoadDataToPI( DataReg[1], 54 );  { 00 11 011 0 }
     (*
      *  00  [=select counter 0]
      *  11  [=write least significant byte first]
      * 011  [=mode 3, divide by n counter]
      *   0  [=16-bits counter]
      *)
 (*
  * load the divisionfactor
  *)
  TimeBaseReg:=0+32;
 (*
  * activate the timebase register
  *)
  LoadDataToPI( ContReg[1], TimeBaseCon         );
  LoadDataToPI( DataReg[1], TimeBaseReg         );
  LoadDataToPI( ContReg[1], WriteTimebaseCon    );
  LoadDataToPI( DataReg[1], lo(wDivisionFactor) );
  LoadDataToPI( DataReg[1], hi(wDivisionFactor) );

  if   rFreq < FreqLoLim
  then begin
        (*
         * extra divider is needed
         *
         * init the timer IC 8254, control word A1,A0=11
         *)
         TimeBaseReg:=3+32;
        (*
         * activate the timebase register
         *)
         LoadDataToPI( ContReg[1], TimeBaseCon      );
         LoadDataToPI( DataReg[1], TimeBaseReg      );
         LoadDataToPI( ContReg[1], WriteTimebaseCon );
         LoadDataToPI( DataReg[1], 118              );  { 01 11 01 0 }
            (*
             *  01  [=select counter 1]
             *  11  [=write least significant byte first]
             * 011  [=mode 3, divide by n counter]
             *   0  [=16-bits counter]
             *
             * load the divisionfactor
             *)
         TimeBaseReg := 1+32;
        (*
         * activate the timebase register
         *)
         LoadDataToPI( ContReg[1], TimeBaseCon      );
         LoadDataToPI( DataReg[1], TimeBaseReg      );
         LoadDataToPI( ContReg[1], WriteTimebaseCon );
         LoadDataToPI( DataReg[1], lo(Div100)       );
         LoadDataToPI( DataReg[1], hi(Div100)       );
        (*
         * select clck input b
         *)
         TimeBaseReg := 16+32
       end  { then }
  else begin
        (*
         * select clck input a
         *)
         TimeBaseReg := 12+32;
       end; { else }

  LoadDataToPI( ContReg[1], TimeBaseCon );
  LoadDataToPI( DataReg[1], TimeBaseReg );

end; { SetTimebase }


procedure SetCalOutput( var rFreq : real );
(*
 * DESCRIPTION
 *   This routine sets the frequency for the square wave generator output
 * INPUT
 *   rFreq      : the frequency to set
 * OUTPUT
 *   rFreq      : the set frequency ( not all frequencies are available )
 *)
const MaxFreq         = 1e+5;
      MinFreq         = 1e+2;
      ClockFreq       = 5e+6;
var   wDivisionFactor : Word;
      rInputFreq      : Real;
begin
  SelectDeviceNrPI( HS508DeviceNrExt );

  rInputFreq := ClockFreq;
  {bepaal wDivisionFactor}
  if   rFreq >  MaxFreq
  then rFreq := MaxFreq;
  if   rFreq <  MinFreq
  then rFreq := MinFreq;

  wDivisionFactor := round( rInputFreq / rFreq );
  rFreq           := rInputFreq / wDivisionFactor;

  TimeBaseReg := 3+32;
  LoadDataToPI( ContReg[1], TimeBaseCon      );
  LoadDataToPI( DataReg[1], TimeBaseReg      );
  LoadDataToPI( ContReg[1], WriteTimebaseCon );
  LoadDataToPI( DataReg[1], 182              );  { 10 11 011 0 }
       (*
        *  10 [=select counter 2]
        *  11 [=write least significant byte first]
        * 011 [=mode 3, divide by n counter]
        *  0  [=16-bits counter]
        *
        * laad het deel tal in counter 2
        *)
  TimeBaseReg := 2+32;
  LoadDataToPI( ContReg[1], TimeBaseCon         );
  LoadDataToPI( DataReg[1], TimeBaseReg         );
  LoadDataToPI( ContReg[1], WriteTimebaseCon    );
  LoadDataToPI( DataReg[1], lo(wDivisionFactor) );
  LoadDataToPI( DataReg[1], hi(wDivisionFactor) );

  TimeBaseReg := 3+32;
  LoadDataToPI( ContReg[1], TimeBaseCon );
  LoadDataToPI( DataReg[1], TimeBaseReg );

end; { SetCalOutput}


procedure ClearSRAM;
(*
 * DESCRIPTION
 *   This routine clears the SRAM in the HS508
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
const HS508MemSize = 32 * 1024;
      MaxFreq      = 25e+6;
var   Index        : integer;
      rFreq        : real;
begin

  SelectDeviceNrPI(HS508DeviceNrExt);

 (*
  * Set clock source to pc
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 0 );
  SetPostCount( HS508MemSize+1 );
  for Index := Ch1 to Ch2 do
  begin
   (*
    * set data to zero
    *)
    LoadDataToPI( ContReg[Index+1], 4 );
    LoadDataToPI( DataReg[Index+1], 1 );
  end;

 (*
  * set fastest timebase
  *)
  rFreq := MaxFreq;
  SetTimebase( rFreq );

 (*
  * set clock source
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 6 );

 (*
  * reset post counter and address counter
  *)
  LoadDataToPI( ContReg[1], 4 );
  LoadDataToPI( DataReg[1], 0 );

 (*
  * switch clock on
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 7 );

end; { ClearSRAM }


procedure GotoSRAMPos( nPos : integer );
(*
 * DESCRIPTION
 *   this routine places a 'pointer' to a requested position in the
 *   HS508 SRAM
 * INPUT
 *   nPos : the requested position
 * OUTPUT
 *   -
 *)
const MaxFreq = 25e+6;
var   rFreq   : real;
begin
  SelectDeviceNrPI(HS508DeviceNrExt);
 (*
  * Set clock source to pc
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 0 );
  SetPostCount( nPos           );

  rFreq := MaxFreq;
  SetTimebase( rFreq );

 (*
  * reset postcounter
  *)
  LoadDataToPI( 0, 4 );
  LoadDataToPI( 1, 0 );

 (*
  * set trig to no trig
  *)
  LoadDataToPI( 0, 7 );
  LoadDataToPI( 1, 2 );

 (*
  * set clock source
  *)
  LoadDataToPI( 0, 3 );
  LoadDataToPI( 1, 2 );

 (*
  * set clock on
  *)
  LoadDataToPI( 0, 3 );
  LoadDataToPI( 1, 3 );

  repeat
    (*
     * just wait
     *)
  until EOC;

 (*
  * Set clock source to pc
  *)
  LoadDataToPI( 0, 3 );
  LoadDataToPI( 1, 0 );

end; { GotoSRAMPos }


procedure ReadDataSRAM( var InstrumentSettings : InstrumentSettingsType;
                        var MeasuredData: MeasurementDataType
                      );
(*
 * DESCRIPTION
 *   This routine reads the measured data from the HS508 SRAM to a data
 *   array in the PC
 * INPUT
 *   InstrumentSettings : some settings for the HS508
 * OUTPUT
 *   MeasuredData       : array containing the measured data
 *)
var nSamplenumber   : integer;
    lHelp           : longint;
    wCount          : Word;
    byCountLo       : byte;
    DeviceAddress_1,
    DeviceAddress_2 : integer;
begin
  DeviceAddress_1 := DeviceAddress + 1;
  DeviceAddress_2 := DeviceAddress + 2;

  SelectDeviceNrPI(HS508DeviceNrExt);

 (*
  * Set clock source to pc
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 0 );

 (*
  * read adres counter
  *)
  byCountLo := ReadDataFromPI( 0 );
  wCount    := ReadDataFromPI( 1 )*256 + byCountLo;

  SelectDeviceNrPI(HS508DeviceNrExt);

  wCount    := wCount and 32767;

  with InstrumentSettings do
    if   byTrigSource = NoTrig
    then lHelp := wCount - lPostSamples - lPreSamples - 6
    else lHelp := wCount - lPostSamples - lPreSamples - 7;

  if   lHelp <  0
  then lHelp := lHelp + 32767;

  GotoSRAMPos( lHelp );

  asm
    MOV DX,DeviceAddress_2
    MOV AL,PPControl
    OUT DX,AL

    {Select HS508DeviceNrExt}
    MOV DX,DeviceAddress
    MOV AL,HS508DeviceNrExt
    OUT DX,AL

    MOV DX,DeviceAddress_2
    MOV AL,PPDevSel OR PPControl
    OUT DX,AL
    MOV AL,PPControl
    OUT DX,AL

    {Load address=0 with data=5 ext device (=inc address)}
    MOV DX,DeviceAddress
    MOV AL,0 {adr}
    OUT DX,AL

    MOV DX,DeviceAddress_2
    MOV AL,PPAdrSel OR PPControl
    OUT DX,AL
    MOV AL,PPControl
    OUT DX,AL

    MOV DX,DeviceAddress
    MOV AL,5 {Data}
    OUT DX,AL

    MOV DX,DeviceAddress_2
    MOV AL,(255-PPIOW) AND PPControl
    OUT DX,AL
    MOV AL,PPControl
    OUT DX,AL
    {END}

  end;  { asm }

  for nSampleNumber := -InstrumentSettings.lPreSamples
                    to  InstrumentSettings.lPostSamples-1
  do begin
     asm
       {Read AD Data CH 1}
       {select address}
       MOV DX,DeviceAddress
       MOV AL,4 {Address 4 is active}
       OUT DX,AL

       MOV DX,DeviceAddress_2
       MOV AL,PPAdrSel+PPIOR+PPControl
       OUT DX,AL
       MOV AL,PPAdrSel OR PPControl
       OUT DX,AL

       {LoData:=(((port[DeviceAddress+1] xor 128) shr 3) and 15);}
       XOR CX,CX
       MOV DX,DeviceAddress_1
       IN  AL,DX
       SHL AX,1
       AND AX,$00F0
       MOV CX,AX

       MOV DX,DeviceAddress
       MOV AL,1 {Data Select Hi Byte}
       OUT DX,AL

       MOV DX,DeviceAddress_1
       IN  AL,DX
       SHL AX,5
       AND AX,$0F00
       OR  CX,AX
       SHR CX,4     { divide by 16 }
       MOV nADVALUE1,CX

       {Read AD Data CH 2}
       {select adres}
       MOV DX,DeviceAddress
       MOV AL,6
       OUT DX,AL

       MOV DX,DeviceAddress_2
       MOV AL,PPAdrSel OR PPIOR OR PPControl
       OUT DX,AL
       MOV AL,PPAdrSel OR PPControl
       OUT DX,AL

       {LoData:=(((port[DeviceAddress+1] xor 128) shr 3) and 15);}
       XOR CX,CX
       MOV DX,DeviceAddress_1
       IN  AL,DX
       SHL AX,1
       AND AX,$00F0
       MOV CX,AX

       MOV DX,DeviceAddress
       MOV AL,1 {Data Select Hi Byte}
       OUT DX,AL

       MOV DX,DeviceAddress_1
       IN  AL,DX
       SHL AX,5
       AND AX,$0F00
       OR  CX,AX
       SHR CX,4     { divide by 16 }
       MOV nADVALUE2,CX

       {verhoog adres teller met 1}
       {LET OP OP DeviceAddress staat al 1 ofwel Adres=1}
       MOV DX,DeviceAddress_2
       MOV AL,(255-PPIOW) AND PPControl {write dummy byte}
       OUT DX,AL

       MOV AL,PPAdrSel OR PPControl
       OUT DX,AL
    end;

    MeasuredData[Ch1][nSampleNumber] := nADValue1;
    MeasuredData[Ch2][nSampleNumber] := nADValue2;

  end;

 (*
  * reset trig channel 1
  *)
  LoadDataToPI(ContReg[2],5);
  LoadDataToPI(DataReg[2],0);
 (*
  * reset trig channel 2
  *)
  LoadDataToPI(ContReg[3],5);
  LoadDataToPI(DataReg[3],0);

  ClearSRAM;

end; { ReadDataSRAM }


procedure SetupForMeasure( var InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION
 *   This routine sets up the HS508 to perform a measurement
 * INPUT
 *   InstrumentSetting : a record containing the settings for the HS508
 * OUTPUT
 *   InstrumentSetting : a record containing the settings for the HS508
 *)
begin

  SetTrigLevel( InstrumentSettings              );
  SetTimebase ( InstrumentSettings.rSampleFreq  );
  SetPostCount( InstrumentSettings.lPostSamples );

  SelectDeviceNrPI(HS508DeviceNrExt);

  if   (MeasureSetting.byTimeBase=1) and ((MeasureSetting.byMeasureMode=Ch1))
  then begin
        (*
         * 50 MSamples only available on channel 1
         *)
         LoadDataToPI( ContReg[2], 4 );
         LoadDataToPI( DataReg[2], 2 );
         LoadDataToPI( ContReg[3], 4 );
         LoadDataToPI( DataReg[3], 0 );
       end
  else begin
        (*
         * all other settings
         *)
         LoadDataToPI( ContReg[2], 4 );
         LoadDataToPI( DataReg[2], 2 );
         LoadDataToPI( ContReg[3], 4 );
         LoadDataToPI( DataReg[3], 2 );
       end;

 (*
  * reset post counter and address counter
  *)
  LoadDataToPI( ContReg[1], 4 );
  LoadDataToPI( DataReg[1], 0 );

 (*
  * Set clock source
  *)
  LoadDataToPI( ContReg[1], 3 );
  LoadDataToPI( DataReg[1], 0 );

 (*
  * Set trigger source
  *)
  LoadDataToPI( ContReg[1], 7 );
  case InstrumentSettings.byTrigSource of
    ExtTrig,
    Ch1Trig : LoadDataToPI( DataReg[1], 0 );
    Ch2Trig : LoadDataToPI( DataReg[1], 1 );
    OrTrig  : LoadDataToPI( DataReg[1], 3 );
    AndTrig : LoadDataToPI( DataReg[1], 4 );
    NoTrig  : LoadDataToPI( DataReg[1], 2 );
    else      LoadDataToPI( DataReg[1], 2 );
  end; { case }

 (*
  * reset post counter and address counter
  *)
  LoadDataToPI( ContReg[1], 4 );
  LoadDataToPI( DataReg[1], 0 );

 (*
  * reset trig channel 1
  *)
  LoadDataToPI( ContReg[2], 5 );
  LoadDataToPI( DataReg[2], 0 );

 (*
  * reset trig channel 2
  *)
  LoadDataToPI( ContReg[3], 5 );
  LoadDataToPI( DataReg[3], 0 );

 (*
  * Set clock sram
  *)
  LoadDataToPI( ContReg[1],  3 );
  LoadDataToPI( DataReg[1], 15 );

end; { SetupForMeasure }


procedure SetEepromSdaScl( SDA, SCL : byte );
(*
 * DESCRIPTION
 *   This routine sets the Serial Data and the Serial Clock of the EEPROM
 * INPUT
 *   SDA : the serial data  bit
 *   SCL : the serial clock bit
 * OUTPUT
 *   -
 *)
begin
  SclSdaByte := 4 + 8;
  if   SCL=1
  then SclSdaByte := SclSdaByte or 1
  else SclSdaByte := SclSdaByte and (255-1);
  if   SDA=1
  then SclSdaByte := SclSdaByte or 2
  else SclSdaByte := SclSdaByte and (255-2);

  SelectDeviceNrPI( HS508DeviceNrInt );

  LoadDataToPI( 0, PiEepromReg );
  LoadDataToPI( 1, SclSdaByte  );

end; { SetEepromSdaScl }


procedure SetEepromSdaSclRead( var SDA: byte; SCL : byte );
(*
 * DESCRIPTION
 *   This routine sets the Eeprom to read from
 * INPUT
 *   SDA        : the Serial DAta bit
 *   SCL        : the Serial Clock bit
 * OUTPUT
 *   SDA        : the Serial DAta bit
 *)
var ByteOut : byte;
begin
  SclSdaByte := 0 + 8;
  if   SCL=1
  then SclSdaByte := SclSdaByte or 1
  else SclSdaByte := SclSdaByte and (255-1);

  SelectDeviceNrPI(HS508DeviceNrInt);

  LoadDataToPI( 0, PiEepromReg );
  LoadDataToPI( 1, SclSdaByte  );

  port[DeviceAddress+2] := PPControl;
  port[DeviceAddress+0] := 0;
  port[DeviceAddress+2] := PPControl or PPAdrSel;
  port[DeviceAddress+2] := PPControl and (255-PPAdrSel);

  SDA := ( ( (port[DeviceAddress+1] shr 7) and 1) Xor 1);

end; { SetEepromSdaSclRead }


procedure StaDataEeprom( ReadWrite : byte );
(*
 * DESCRIPTION
 *   This routine generates a start puls for the i2c communication
 * INPUT
 *   ReadWrite : indicates reading / writing
 * OUTPUT
 *   -
 *)
const EepromAddress = 160;     { the I2C address of the EEprom }
var   n, m          : integer;
      byInBit,
      byOutBit      : byte;
      byEepromData  : byte;
begin

  SetEepromSdaScl( 1, 1 );
  SetEepromSdaScl( 0, 1 );
  SetEepromSdaScl( 0, 0) ;

  byEepromData := EepromAddress + ReadWrite;

  for n := 7 downto 0 do
  begin
    m:=round(exp(n*ln(2)));

    if   ( byEepromData-m >= 0 )
    then begin
           byEepromData := byEepromData - m;
           byOutBit     := 1;
         end
    else byOutBit := 0;

    SetEepromSdaScl( byOutBit, 0 );
    SetEepromSdaScl( byOutBit, 1 );
    SetEepromSdaScl( byOutBit, 0 );
  end; { for }

 (*
  * Acknowledge bit
  *)
  SetEepromSdaSclRead( byInBit, 0 );
  SetEepromSdaSclRead( byinBit, 1 );
  SetEepromSdaScl    ( 0, 0       );

end; { StaDataEeprom }


procedure StoDataEeprom;
(*
 * DESCRIPTION
 *   This routine generates a stopbit for the i2c communication
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin

  SetEepromSdaScl( 0, 1 );
  SetEepromSdaScl( 1, 1 );

end; { StoDataEeprom }


procedure WriteDataEeprom( byEepromData : byte );
(*
 * DESCRIPTION
 *   This routine writes a byte to the eeprom
 * INPUT
 *   by EepromData : the byte to write
 * OUTPUT
 *
 *)
var n, m     : integer;
    byInBit,
    byOutBit : byte;
begin

  for n := 7 downto 0 do
  begin
    m := round( exp(n*ln(2)));
    if   ( byEepromData-m >= 0 )
    then begin
           byEepromData := byEepromData - m;
           byOutBit := 1;
         end
    else byOutBit := 0;

    SetEepromSdaScl( byOutBit, 0 );
    SetEepromSdaScl( byOutBit, 1 );
    SetEepromSdaScl( byOutBit, 0 );
  end;

 (*
  * Acknowledge bit
  *)
  SetEepromSdaSclRead( byInBit, 0 );
  SetEepromSdaSclRead( byInBit, 1 );
  SetEepromSdaScl    ( 0,       0 );

end; { WriteDataEeprom }


procedure ReadDataEeprom( var byEepromData : byte );
(*
 * DESCRIPTION
 *   This routine reads a byte from the eeprom
 * INPUT
 *   -
 * OUTPUT
 *   byEepromData : the read byte
 *)
var Index   : integer;
    byInBit : byte;
begin
  byEepromData := 0;
  for Index := 7 Downto 0 do
  begin
    SetEepromSdaSclRead( byInBit, 0 );
    SetEepromSdaSclRead( byInBit, 1 );
    byEepromData := ( byInBit shl Index ) + byEepromData;
    SetEepromSdaSclRead( byInBit, 0 );
  end;

 (*
  * Acknowledge bit
  *)
  SetEepromSdaScl( 1, 0 );
  SetEepromSdaScl( 1, 1 );
  SetEepromSdaScl( 1, 0 );

end; { ReadDataEeprom }


procedure ReadDataFromEeprom( byAddress : byte; var byData : byte );
(*
 * DESCRIPTION
 *   This routine reads one byte from a specified address in the eeprom
 * INPUT
 *   byAddress : the address to read from
 * OUTPUT
 *   byData    : the data that is read from the eeprom
 *)

begin

  StaDataEeprom  ( 0         );    { write data to Eeprom  }
  WriteDataEeprom( byAddress );    { Address               }
  StoDataEeprom;

  StaDataEeprom ( 1          );    { Read data from Eeprom }
  ReadDataEeprom( byData     );    { Data                  }
  StoDataEeprom;

end; { ReadDataFromEeprom }


procedure ReadEEpromOffset( nCh, nSetting : integer );
(*
 * DESCRIPTION
 *   This routine reads one offset calibration factor from the eeprom.
 *   the calfactor is written directly in the global offset calfactor array
 * INPUT
 *   nCh        : the HS508 channel for the calfactor
 *   nSetting   : the attenuator setting for the calfactor
 * OUTPUT
 *   -
 *)
var nValue    : integer;
    byHiValue,
    byLoValue : byte;
    nAddress  : integer;
begin

  case nCh of
    1 : nAddress := EEpromAdrOffsetCh1 + (nSetting-1)*2;
    2 : nAddress := EEpromAdrOffsetCh2 + (nSetting-1)*2;
    3 : nAddress := EEpromAdrOffsetCh3 + (nSetting-1)*2;
  end; { case }

  ReadDataFromEeprom( nAddress,    byLoValue );
  ReadDataFromEeprom( nAddress+1 , byHiValue );
  nValue := byHivalue;
  nValue := nValue shl 8;
  nValue := nValue + byLoValue;
  ValueRef[nCh][nSetting] := ((2.8-0.3)/2) * ((nValue-512)/512);

end; { ReadEEpromOffset }


procedure ReadEEpromGain( nCh, nSetting : integer );
(*
 * DESCRIPTION
 *   This routine reads one gain calibration factor from the eeprom.
 *   the calfactor is written directly in the global gain calfactor array
 * INPUT
 *   nCh        : the HS508 channel for the calfactor
 *   nSetting   : the attenuator setting for the calfactor
 * OUTPUT
 *   -
 *)
var nValue     : integer;
    byHiValue,
    byLoValue  : byte;
    nAddress   : integer;
begin

  case nCh of
    1 : nAddress := EEpromAdrGainCh1 + (nSetting-1)*2;
    2 : nAddress := EEpromAdrGainCh2 + (nSetting-1)*2;
    3 : nAddress := EEpromAdrGainCh3 + (nSetting-1)*2;
  end; { case }

  ReadDataFromEeprom( nAddress,   byLovalue );
  ReadDataFromEeprom( nAddress+1, byHivalue );
  nValue := byHivalue;
  nValue := nValue shl 8;
  nValue := nValue + byLoValue;
  ValueDA[nCh][nSetting] := nValue;

end; { ReadEEpromGain }


function HardwarePresent : boolean;
(*
 * DESCRIPTION
 *   This routine checks whether the HS508 hardware is present.
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue:       True  if hardware is present
 *                      False if hardware is not present
 *)
var   index        : integer;
      nValue       : integer;
      byEepromData : byte;
begin

  for Index := 1 to 3 do
  begin
    case Index of
      1 : DeviceAddress := LPT1Address;
      2 : DeviceAddress := LPT2Address;
      3 : DeviceAddress := LPT3Address;
    end; { case }
    ReadDataFromEeprom( EEpromTypeId, byEepromData );
    if   byEepromData = IdDevice
    then begin
           HardwarePresent := true;
           exit;
         end; { if then }
  end; { for }
  HardwarePresent := false;

end;


function GetSerialNumber : longint;
(*
 * DESCRIPTION
 *   This routine reads the instrument serial number from the eeprom
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue: the instrument serialnumber
 *)
var wSerialNr     : word;
    byLoSerialNr,
    byMiSerialNr,
    byHiSerialNr  : byte;
begin
  if HardwarePresent
  then begin
         ReadDataFromEeprom( EEpromSerialNr1+1, byLoSerialNr );
         ReadDataFromEeprom( EEpromSerialNr1+2, byMiSerialNr );
         ReadDataFromEeprom( EEpromSerialNr1+3, byHiSerialNr );
         wSerialNr       := byMiSerialNr shl 8;
         GetSerialNumber := byLoSerialNr + wSerialNr;
       end
  else begin
         GetSerialNumber := -1;
       end;

end; { GetSerialNumber }



procedure ReadCalibrateData;
(*
 * DESCRIPTION
 *   This routine reads the calibration values from the HS508 Eeprom.
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
var nCh : integer;
begin

 for nCh := Ch1 to Ch2 do
 begin
    ReadEEpromGain( nCh,  1 );
    ReadEEpromGain( nCh,  2 );
    ReadEEpromGain( nCh,  3 );
    ReadEEpromGain( nCh,  4 );
    ReadEEpromGain( nCh,  5 );
    ReadEEpromGain( nCh,  6 );
    ReadEEpromGain( nCh,  7 );
    ReadEEpromGain( nCh,  8 );
    ReadEEpromGain( nCh,  9 );
    ReadEEpromGain( nCh, 10 );
    ReadEEpromGain( nCh, 11 );
    ReadEEpromGain( nCh, 12 );

    ReadEEpromOffset( nCh,  1 );
    ReadEEpromOffset( nCh,  2 );
    ReadEEpromOffset( nCh,  3 );
    ReadEEpromOffset( nCh,  4 );
    ReadEEpromOffset( nCh,  5 );
    ReadEEpromOffset( nCh,  6 );
    ReadEEpromOffset( nCh,  7 );
    ReadEEpromOffset( nCh,  8 );
    ReadEEpromOffset( nCh,  9 );
    ReadEEpromOffset( nCh, 10 );
    ReadEEpromOffset( nCh, 11 );
    ReadEEpromOffset( nCh, 12 );
  end;

end; { ReadCalibrateData }


procedure CalibrateInstrument( var bCalError : boolean );
(*
 * DESCRIPTION
 *   This routine calibrates the HS508
 * INPUT
 *   -
 * OUTPUT
 *   bCalError : flag indicating whether calibration succeeded
 *)
begin

  ReadCalibrateData;

 (*
  * The actual instrument calibration is done in the factory, this routine
  * just reads the calibration factors from the EEprom.
  * This calibration is always succesfull, flag is only present to maintain
  * similarity with software for other devices.
  *)
  bCalError := False;

end; { CalibrateInstrument }


procedure InitInstrument;
(*
 * DESCRIPTION
 *   This routine initializes the PC to use the HS508 and
 *   the HS508 to perform measurements
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
var nCh       : integer;
    nSetting  : integer;
    bCalError : boolean;
begin

 (*
  * Save the original timer interrupt vector of the PC
  * and assign a new interrupt routine to the vector
  *)
  GetIntVec( $1C,  int1Csave    );
  SetIntVec( $1C, @TimeOutTimer );

  ExitSave := ExitProc;
  ExitProc := @ProgramExit;


 (*
  * Chech whether a HS508 is present
  *)
  if   not HardwarePresent
  then begin
         write( strText2 );
         exit
       end;

 (*
  * if this point is achieved, a HS508 is present
  *
  * set it up
  *)
  PgaSwitch[1][ 1]   := Pga1Att1;
  PgaSwitch[1][ 2]   := Pga1Att1;
  PgaSwitch[1][ 3]   := Pga1Att1;
  PgaSwitch[1][ 4]   := Pga1Att1;
  PgaSwitch[1][ 5]   := Pga1Att1;
  PgaSwitch[1][ 6]   := Pga1Att1;
  PgaSwitch[1][ 7]   := Pga1Att10;
  PgaSwitch[1][ 8]   := Pga1Att10;
  PgaSwitch[1][ 9]   := Pga1Att10;
  PgaSwitch[1][10]   := Pga1Att100;
  PgaSwitch[1][11]   := Pga1Att100;
  PgaSwitch[1][12]   := Pga1Att100;

  PgaSwitch[2][ 1]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 2]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 3]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 4]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 5]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 6]   := Pga2Att1   + Pga2on;
  PgaSwitch[2][ 7]   := Pga2Att10  + Pga2on;
  PgaSwitch[2][ 8]   := Pga2Att10  + Pga2on;
  PgaSwitch[2][ 9]   := Pga2Att10  + Pga2on;
  PgaSwitch[2][10]   := Pga2Att100 + Pga2on;
  PgaSwitch[2][11]   := Pga2Att100 + Pga2on;
  PgaSwitch[2][12]   := Pga2Att100 + Pga2on;

  PgaSwitchValue[ 1] :=  20e-3;
  PgaSwitchValue[ 2] :=  40e-3;
  PgaSwitchValue[ 3] :=  80e-3;
  PgaSwitchValue[ 4] := 200e-3;
  PgaSwitchValue[ 5] := 400e-3;
  PgaSwitchValue[ 6] := 800e-3;
  PgaSwitchValue[ 7] :=   2e+0;
  PgaSwitchValue[ 8] :=   4e+0;
  PgaSwitchValue[ 9] :=   8e+0;
  PgaSwitchValue[10] :=  20e+0;
  PgaSwitchValue[11] :=  40e+0;
  PgaSwitchValue[12] :=  80e+0;


  ClearSRAM;

  for nCh := Ch1 to Ch2 do
    for nSetting := 1 to MaxSetting do
      ValueRef[nCh][nSetting] := 0.0;
  for nCh := 1 to 3 do
    for nSetting := 1 to MaxSetting do
      ValueDA[nCh][nSetting] := 127;

  CalibrateInstrument( bCalError );

end; { InitInstrument }



procedure StartMeasurement(     MeasureSetting : MeasurementSettingsType;
                            var MeasureData    : MeasurementDataType
                          );
(*
 * DESCRIPTION
 *   This routine performs one measurement with the HS508
 * INPUT
 *   MeasureSetting     : the mesurement settings
 * OUTPUT
 *   MeasureData        : the measured data
 *)
var lSwitchSetting    : longint;
    InstrumentSetting : InstrumentSettingsType;
begin
  with MeasureSetting do
  begin

    case byMeasureMode of
      Cmp,
      Ch1  : if   byTimebase = 1
             then lSwitchSetting := PgaSwitch[1][bySwitch[1]] +
                                    PgaSwitch[2][bySwitch[2]] -
                                    Pga1on + Pga2on
             else lSwitchSetting := PgaSwitch[1][bySwitch[1]] +
                                    PgaSwitch[2][bySwitch[2]];
      Ch2,
      Chop,
      Add   : lSwitchSetting := PgaSwitch[1][bySwitch[1]] +
                                PgaSwitch[2][bySwitch[2]];
    end; { case }

    SetDa( 0, (rDcLevel[1]/PgaSwitchValue[bySwitch[1]]) * DcOffsetGain + ValueRef[1][bySwitch[1]]);
    SetDa( 1, (rDcLevel[2]/PgaSwitchValue[bySwitch[2]]) * DcOffsetGain + ValueRef[2][bySwitch[2]]);
    SetDa( 2, (-512 + ValueDA[1][bySwitch[1]]) * (2.5/1024) ); {Gain Ch1}
    SetDa( 3, (-512 + ValueDA[2][bySwitch[2]]) * (2.5/1024) ); {Gain Ch2}

    SetPga( bCoupling[Ch1], bCoupling[Ch2], lSwitchSetting );

    InstrumentSetting.rSampleFreq      := rSampleFreq;
    InstrumentSetting.lPostSamples     := lPostSamples;
    InstrumentSetting.lPreSamples      := lPreSamples;
    InstrumentSetting.byTrigHys        := byTrigHys;
    InstrumentSetting.byTrigSource     := nTrigSource;
    InstrumentSetting.byTrigLevel[Ch1] := byTrigLevel[Ch1];
    InstrumentSetting.byTrigLevel[Ch1] := byTrigLevel[Ch1];
    InstrumentSetting.bTrigSlope       := bTrigSlope;

  end; { with }

  SetupForMeasure( InstrumentSetting );

  asm
    CLI             { disable interrupts    }
  end;
  lTimerCount := 0; { clear timeout counter }
  asm
    STI             { enable interrupts     }
  end;

  repeat
   (*
    * This loop is to wait for the End of Conversion signal
    * In this loop, the timeout counter is monitored.
    * On interruptbase this counter is increased 18.2 times per second.
    * If the timeout value is reached, a start of measurement is forced
    * by setting the Trigger bit.
    * Waiting continues until an End Of Conversion is detected
    *)
    if   (lTimerCount >= MeasureSetting.lTimeOut)
    then SetTrigbit;
  until EOC;

  if EOC
  then begin
         ReadDataSram( InstrumentSetting, MeasureData );
       end;

end; { StartMeasurement }
