(*
 * This file contains TP112 specific routines and consts
 * It is to be included in the main source file
 *)

const LPT1Address      = $378;
      LPT2Address      = $278;
      LPT3Address      = $3BC;
      TP208Address     = $300;
      TP508Address     = $300;
      TP112Address     = $300;

      ContReg          : array[1..3] of Byte = ( 0, 4, 6 );
      DataReg          : array[1..3] of Byte = ( 1, 5, 7 );

      PostLoCon        = 0;
      PostHiCon        = 1;
      OutputCon        = 2;
      TimebaseCon      = 3;
      ModeCon          = 4;
      ResetCon         = 5;
      CountCon         = 6;
      WriteTimebaseCon = 7;

      Level1ConAd      : array[1..2] of Byte = ( 0, 0 );
      Level2ConAd      : array[1..2] of Byte = ( 1, 1 );
      TrigLevelConAd   : array[1..2] of Byte = ( 2, 2 );
      PGAConAd         : array[1..2] of Byte = ( 3, 3 );
      DateZeroConAd    : array[1..2] of Byte = ( 4, 4 );
      IRQConAd         : array[1..2] of Byte = ( 5, 5 );
      TrigResetConAd   : array[1..2] of Byte = ( 6, 6 );

      Level1RegAd      : array[1..2] of Byte = ( 0, 0 );
      Level2RegAd      : array[1..2] of Byte = ( 0, 0 );
      TrigLevRegAd     : array[1..2] of Byte = ( 0, 0 );
      PgaRegAd         : array[1..2] of Byte = ( 0, 0 );
      TrigResetRegAd   : array[1..2] of Byte = ( 0, 0 );

      PostLoReg        : byte = 0;
      PostHiReg        : byte = 0;
      OutputReg        : byte = 0;
      TimebaseReg      : byte = 0;
      ModeReg          : byte = 0;
      ResetReg         : byte = 0;
      ContrReg         : byte = 0;
      WriteTimebaseReg : byte = 0;

      PGASwitchTime    = 10;



      Pga1250mV        = 1;
      Pga2500mV        = 2;
      Pga5V            = 3;
      Pga10V           = 4;
      Pga20V           = 5;

      MaxSetting       = 5;      { Hightest input range setting available }
      MinSetting       = 1;      { Lowest   input range setting available }

      CristalFreq      = 4e+6;
      MaxSamplingFreq  = 1e+6;

      DeviceMaxValue   = 4095;   { 12 bits instrument }

      DeviceString     = 'TP112';



{$F+}  { force FAR function calls }
procedure TimeOutTimer;
(*
 * DESCRIPTION:
 *   This interrupt routine is called 18.2 times per second. It is used to
 *   to determine whether the trigger system is timed out and a start of
 *   measurement is to be forced
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
interrupt;
begin

  inc( lTimerCount );

end; { TimeOutTimer }

procedure ProgramExit;
(*
 * DESCRIPTION
 *   This routine is called at termination of the program, by the
 *   Borland Pascal Runtime library
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin
 (*
  * restore original exit procedure
  *)
  ExitProc := ExitSave;

 (*
  * restore normal timer interrupt routine
  *)
  SetIntvec($1c, Int1Csave );
  
end;
{$F-}  { use NEAR function calls }


{*****************************************************************************}
{******************** INT_ON_OFF        **************************************}
{*****************************************************************************}
procedure IntOnOff( bOn : boolean );
var i      : integer;
    byHelp : byte;
begin

  if   bOn
  then byHelp := 255  {1111 1111}
  else byHelp := 127; {0111 1111}

  for index := Ch1 to Ch2 do
  begin
    port[DeviceAddress+ContReg[Index+1]] := TrigLevelConAd[Index];
    TrigLevRegAd[Index]                  := TrigLevRegAd[Index] and 127;
    TrigLevRegAd[Index]                  := TrigLevRegAd[Index] or  byHelp;
    port[Deviceaddress+DataReg[Index+1]] := TrigLevRegAd[Index];
  end;

end; { IntOnOff }

function nClip( nLoLim, nValue, nHiLim : integer ) : integer;
(*
 * DESCRIPTION
 *   This function clips nValue to nLoLim and nHiLim
 * INPUT
 *   nValue      : the value to clip
 *   nLoLim      : the lower limit to clip to
 *   nHiLim      : the higher limit to clip to
 * OUTPUT
 *   returnvalue : the clipped value
 *)
begin

  if   nValue >  nHiLim
  then nValue := nHiLim;

  if   nValue <  nLoLim
  then nValue := nLoLim;

  nClip := nValue;

end; { nClip }


procedure SetPga( byCh : byte; bCouplingAC: boolean; bySetting : byte );
(*
 * DESCRIPTION
 *   This routine sets the coupling switches and the
 *   attenuator switches either Ch1 or Ch2
 * INPUT
 *   byCh        : indicates which channel to change
 *   bCouplingAC : the requested state for the coupling switch
 *   bySetting   : a parameter for the attenuator switches
 * OUTPUT
 *   -
 *)
 begin

  byCh      := nClip( Ch1, byCh, Ch2 );
  bySetting := PgaSwitch[byCh][nClip( MinSetting, bySetting, MaxSetting )];

  if   not bCouplingAC
  then bySetting := bySetting or 8;

  PgaRegAd[byCh] := bySetting;

  port[DeviceAddress+ContReg[byCh+1]] := PgaConAd[byCh];
  port[DeviceAddress+DataReg[byCh+1]] := PgaRegAd[byCh];

end; { SetPga }


procedure SetRef( nValue1, nValue2 : integer);
(*
 * DESCRIPTION
 *   This routine is empty
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin
end; { SetRef }


procedure SetDa( nCh : integer; rValue : real );
begin
end; { SetDA }


function EOC : boolean;
(*
 * DESCRIPTION
 *   This routine reads the End Of Conversion bit
 * INPUT
 *   -
 * OUTPUT
 *   Return value : the status of the End Of Conversion bit
 *)
begin

  EOC := (port[DeviceAddress+2] and 1) = 1;

end; {EOC}



procedure SetPostCount( wPostCount : word );
(*
 * DESCRIPTION
 *   This routine sets the post counter
 * INPUT
 *   wPostCount : the new value for the post counter
 * OUTPUT
 *   -
 *)
begin

  PostLoReg                      := lo(wPostCount);
  PostHiReg                      := hi(wPostCount);
  port[DeviceAddress+ContReg[1]] := PostLoCon;
  port[DeviceAddress+DataReg[1]] := PostLoReg;
  port[DeviceAddress+ContReg[1]] := PostHiCon;
  port[DeviceAddress+DataReg[1]] := PostHiReg;

end; { SetPostCount }


procedure SetTriglevel( InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION
 *   This routine sets the trigger level for both channels
 * INPUT
 *   InstrumentSettings : a structure holding the instrument settings
 * OUTPUT
 *   -
 *)
var nCh   : integer;
    nHelp : integer;
begin

 (*
  * do both channels
  *)
  with InstrumentSettings do
    for nCh := Ch1 to Ch2 do
    begin
      if   bTrigSlope = PosSlope
      then begin
             Level1RegAd[nCh]  := nClip( 0, byTrigLevel[nCh]-byTrigHys, 255 );
             Level2RegAd[nCh]  := nClip( 0, byTrigLevel[nCh],           255 );
             TrigLevRegAd[nCh] := 5; {0000 0101}
           end
      else begin
             Level1RegAd[nCh]  := nClip( 0, byTrigLevel[nCh]+byTrigHys, 255 );
             Level2RegAd[nCh]  := nClip( 0, byTrigLevel[nCh],           255 );
             TrigLevRegAd[nCh] := 6; {0000 0101}
           end;

      port[DeviceAddress+ContReg[nCh+1]] := TrigLevelConAd[nCh];
      port[DeviceAddress+DataReg[nCh+1]] := TrigLevRegAd[nCh];

      port[DeviceAddress+ContReg[nCh+1]] := Level1ConAd[nCh];
      port[DeviceAddress+DataReg[nCh+1]] := Level1RegAd[nCh];
      port[DeviceAddress+ContReg[nCh+1]] := Level2ConAd[nCh];
      port[DeviceAddress+DataReg[nCh+1]] := Level2RegAd[nCh];

    end; { for }

end; { SetTrigLevel }


procedure SetTrigbit( InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION
 *   This routine sets the trigger bit, to force a start of measurement
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
var nCh  : integer;
    nHelp : integer;
begin
  port[DeviceAddress+0] := 4;
  port[DeviceAddress+1] := 128 + 15;
end; { SetTrigBit }


procedure SetTimebase( var rFreq : real );
(*
 * DESCRIPTION
 *   This routine sets the timebase of the TP112 to the requested sampling
 *   frequency
 * INPUT
 *   rFreq : the frequency to set
 * OUTPUT
 *   rFreq : the set frequency (input is clipped)
 * CAVEAT
 *  This routine has several exit points
 *)
const CristalFreq     =  4e+6; { the cristal frequency on the TP112         }
      MaxFreq         =  1e+6; { maximum sampling frequency                 }
      FreqDiv100      =  4e+4; { cristal frequency / 100                    }
      FreqLoLim       =    62; { lower limit frequency to be generated with }
                               { a 16-bits divider (4e6/65535)              }
      Div100          =   100; { division factor for extra divider          }
var   wDivisionFactor : word;
      rInputFreq      : real;
      byTimebaseReg   : byte;
begin

 (*
  * the frequencies are generated by a timer chip
  *)

  rInputFreq := CristalFreq;

  if   rFreq >  MaxFreq
  then rFreq := MaxFreq;

  if   rFreq < FreqLoLim
  then rInputFreq := FreqDiv100;

  wDivisionFactor := round( rInputFreq / rFreq );
  rFreq           := rInputFreq / wDivisionFactor;

 (*
  * init the timer IC 8254, control word A1,A0 =11
  *)
  byTimebaseReg                  := 3;
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := 54;  { 00 11 011 0 }
      (*
       * 00  [ = select counter 0                   ]
       * 11  [ = write least significant byte first ]
       * 011 [ = mode 3, divide by n counter        ]
       * 0   [ = 16-bits counter                    ]
       *)
  byTimebaseReg                  := 0;
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := TimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := lo(wDivisionFactor);
  port[DeviceAddress+DataReg[1]] := hi(wDivisionFactor);

  if   rFreq < FreqLoLim
  then begin
        (*
         * extra divider is needed
         *)
         byTimebaseReg                  := 3;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
         port[DeviceAddress+DataReg[1]] := 118;  { 01 11 011 0 }
             (*
              * 01  [ =select counter 1                   ]
              * 11  [ =write least significant byte first ]
              * 011 [ =mode 3, divide by n counter        ]
              * 0   [ =16-bits counter                    ]
              *)
         byTimebaseReg                  := 1;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
         port[DeviceAddress+DataReg[1]] := lo(Div100);
         port[DeviceAddress+DataReg[1]] := hi(Div100);

        (*
         * select clock input b
         *)
         byTimebaseReg := 8;
       end
  else begin
        (*
         * can be made with one divider, so no extra divider needed
         * select clock input a
         *)
         byTimebaseReg := 0;
       end;

  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := TimebaseReg;

end; { SetTimebase }


procedure SetCalOutput( var rFreq : real );
begin
end; { SetCalOutput }



procedure ClearSRAM;
(*
 * DESCRIPTION
 *   This routine clears the SRAM
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
const TP112MemSize = 32 * 1024;
      MaxFreq      = 1e+6;
var   nCh   : integer;
      rFreq : real;
begin

  SetPostCount( TP112MemSize+1 );

  for nCh := Ch1 to Ch2 do
  begin
   (*
    * Set data/zero to zero
    *)
    port[DeviceAddress+ContReg[nCh+1]] := 4;
    port[DeviceAddress+DataReg[nCh+1]] := 1;
  end;

 (*
  * select fastest timebase
  *)
  rFreq := MaxFreq;
  SetTimebase( rFreq );

 (*
  * Set ClockSource
  *)
  port[DeviceAddress+ContReg[1]] :=   4;
  port[DeviceAddress+DataReg[1]] := 142;

 (*
  * reset post counter and address counter
  *)
  port[DeviceAddress+ContReg[1]] := 5;
  port[DeviceAddress+DataReg[1]] := 0;
  delay(1);

 (*
  * Set clock on
  *)
  port[DeviceAddress+ContReg[1]] :=   4;
  port[DeviceAddress+DataReg[1]] := 143;

end; { ClearSRAM }


procedure GotoSRAMPos( wPos : word );
(*
 * DESCRIPTION
 *   This routine places a 'pointer' to a requested position in the
 *   TP112 SRAM
 * INPUT
 *   nPos : the requested position
 * OUTPUT
 *   -
 *)
const MaxFreq = 1e+6;
var   rFreq   : real;
begin

  port[DeviceAddress+ContReg[1]] := PostLoCon;
  port[DeviceAddress+DataReg[1]] := lo(wPos);
  port[DeviceAddress+ContReg[1]] := PostHiCon;
  port[DeviceAddress+DataReg[1]] := hi(wPos);

  rFreq := MaxFreq;
  SetTimebase( rFreq );

 (*
  * reset post counter
  *)
  port[DeviceAddress+0] :=   5;
  port[DeviceAddress+1] :=   0;

 (*
  *  set clock source
  *)
  port[DeviceAddress+0] :=   4;
  port[DeviceAddress+1] := 129;

 (*
  * set clock on
  *)
  port[DeviceAddress+0] :=   4;
  port[DeviceAddress+1] := 131;

  delay(1);
  repeat
   (*
    * nothing, just wait
    *)
  until EOC;

 (*
  * Set clock off
  *)
  port[DeviceAddress+0] := 4;
  port[DeviceAddress+1] := 0;

  port[DeviceAddress+ContReg[1]] := PostLoCon;
  port[DeviceAddress+DataReg[1]] := PostLoReg;
  port[DeviceAddress+ContReg[1]] := PostHiCon;
  port[DeviceAddress+DataReg[1]] := PostHiReg;

end; { GotoSRAMPos }



procedure ReadDataSRAM(     InstrumentSettings : InstrumentSettingsType;
                        var Data               : MeasurementDataType
                      );
(*
 * DESCRIPTION
 *   This routine reads the measured data from SRAM of the TP112 to a
 *   data array
 * INPUT
 *   InstrumentSettings : holds some settings of the TP112
 * OUTPUT
 *   Data               : array containing the measured data
 *)
var wLo,
    wHi       : word;
    nSampleNr : integer;
    lHelp     : longint;
    wCount    : word;
begin

 (*
  * read adres counter
  *)
  wCount := (256*port[DeviceAddress+1] + port[DeviceAddress]) and 32767;

 (*
  * set clock source
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := 0;

  with InstrumentSettings do
  begin
    GotoSRAMPos(wCount-(lPostSamples+lPreSamples)-5);

   (*
    * Set clock source
    *)
    port[DeviceAddress+ContReg[1]] :=   4;
    port[DeviceAddress+DataReg[1]] := 128;

    for nSampleNr := -lPreSamples to lPostSamples-1 do
    begin
     (*
      * Read data
      *)
      wLo                := port[DeviceAddress+4];
      wHi                := port[DeviceAddress+5] and 15;
      Data[1][nSampleNr] := wLo + 256*wHi;
      wLo                := port[DeviceAddress+6];
      wHi                := port[DeviceAddress+7] and 15;
      Data[2][nSampleNr] := wLo + 256*wHi;

     (*
      * increase address counter
      *)
      port[DeviceAddress+ContReg[1]] := 6;
      port[DeviceAddress+DataReg[1]] := 0;
    end; { for }
  end; { with }

 (*
  * reset trig channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 6;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * reset trig channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 6;
  port[DeviceAddress+DataReg[3]] := 0;

  ClearSRAM;

end;  { ReadDataSRAM }


procedure SetupForMeasurement( var InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION
 *   This routine sets up the TP112 to perform a measurement
 * INPUT
 *   InstrumentSettings : a records with the settings to measure according to
 * OUTPUT
 *   InstrumentSettings : a records with the settings measured according to
 *)
var byHelp : byte;
begin

  SetTriglevel(InstrumentSettings);
  with InstrumentSettings do
  begin
    SetTimebase(rSampleFreq);

    case byTrigSource of
      Ch1    : byHelp :=   0; { 0000 0000 }
      Ch2    : byHelp :=  64; { 0100 0000 }
      NoTrig : byHelp := 128; { 1000 0000 }
      else     byHelp := 192; { 1100 0000 }
    end; { case }

    SetPostCount(lPostSamples);
  end; { with }

 (*
  * Set data/zero to data channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 4;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * Set data/zero to data channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 4;
  port[DeviceAddress+DataReg[3]] := 0;

 (*
  * reset post counter and address counter
  *)
  port[DeviceAddress+ContReg[1]] := 5;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * Set clock source
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := byHelp+14;

 (*
  * reset post counter and address counter
  *)
  port[DeviceAddress+ContReg[1]] := 5;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * reset trig channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 6;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * reset trig channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 6;
  port[DeviceAddress+DataReg[3]] := 0;

 (*
  * Set clock on
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := byHelp + 15;



end; { SetupForMeasurement }


procedure ReadDigitalInputs( var InstrumentSettings : InstrumentSettingsType;
                                 Data               : MeasurementDataType );
(*
 * DESCRIPTION
 *   This routine reads the digital input data from SRAM of the TP112 to a
 *   data array
 * INPUT
 *   InstrumentSettings : holds some settings of the TP112
 * OUTPUT
 *   Data               : array containing the 'measured' data, in Ch1
 *)
var wLo,
    wHi       : word;
    nSampleNr : integer;
    lHelp     : longint;
    wCount    : word;
begin

 (*
  * read adres counter
  *)
  wCount := (256*port[DeviceAddress+1] + port[DeviceAddress]) and 32767;

 (*
  * set clock source
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := 0;

  with InstrumentSettings do
  begin
    GotoSRAMPos(wCount-(lPostSamples+lPreSamples)-5);

   (*
    * Set clock source
    *)
    port[DeviceAddress+ContReg[1]] :=   4;
    port[DeviceAddress+DataReg[1]] := 128;

    for nSampleNr := -lPreSamples to lPostSamples-1 do
    begin
     (*
      * Read the digital input data.
      *
      * The TP112 is a 2 channel 12-bit instrument, with additionally 8
      * digital inputs. The data for the digital inputs is stored in the
      * higher four bits of the SRAM of both channels. That space is
      * available, since the SRAM is 16 bits wide and the TP112 measures
      * 12 bits.
      *)
      wLo                  := port[DeviceAddress+5] and $F0; { 1111 0000 }
      wHi                  := port[DeviceAddress+7] and $F0; { 1111 0000 }
      Data[Ch1][nSampleNr] := (wLo shr 4) + wHi;

     (*
      * increase address counter
      *)
      port[DeviceAddress+ContReg[1]] := 6;
      port[DeviceAddress+DataReg[1]] := 0;
    end; { for }
  end; { with }

 (*
  * reset trig channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 6;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * reset trig channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 6;
  port[DeviceAddress+DataReg[3]] := 0;

  ClearSRAM;

end; { ReadSwitchSetting }


procedure WriteDigitalOutputs( OutByte : byte );
(*
 * DESCRIPTION
 *   This routine writes a value to the digital outputs of the TP112
 * INPUT
 *   OutByte : The value that has to be written to the digital outputs
 * OUTPUT
 *   -
 *)
begin
  port[DeviceAddress+ContReg[1]] := OutputCon;
  port[DeviceAddress+DataReg[1]] := OutByte;
end; { WriteDigitalOutputs }


function HardwarePresent : boolean;
(*
 * DESCRIPTION
 *   This routine checks whether the TP112 hardware is present
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue : True  - hardware is present
 *               : False - hardware is absent
 *)
var Index  : Integer;
    nValue : Integer;
begin

  for Index := 1 to 10 do
    nValue := port[DeviceAddress+3];
  if   nValue = 0
  then HardwarePresent := true
  else HardwarePresent := false;

end;

procedure InitInstrument;
(*
 * DESCRIPTION
 *   This routine initializes the PC to use the TP112
 *   and the TP112 to perform measurements
 * INPUT
 *   -
 * OUTPUT
 *   -
 *)
begin

 (*
  * Save the original Timer interrupt vector of the PC (interrupt $1C)
  * and assign a new routine to it
  *)
  GetIntVec( $1C,  Int1Csave    );
  SetIntVec( $1C, @TimeOutTimer );

  ExitSave := ExitProc;
  ExitProc := @ProgramExit;


  DeviceAddress := TP112Address;

 (*
  * Check for a TP112
  *)
  if   not HardwarePresent
  then begin
         write( strText2 );
         exit;
       end;

 (*
  * apparently the hardware is present
  *)
  PgaSwitch[Ch1][Pga1250mV] := 4;
  PgaSwitch[Ch1][Pga2500mV] := 3;
  PgaSwitch[Ch1][Pga5V    ] := 2;
  PgaSwitch[Ch1][Pga10V   ] := 1;
  PgaSwitch[Ch1][Pga20V   ] := 0;

  PgaSwitch[Ch2][Pga1250mV] := 4;
  PgaSwitch[Ch2][Pga2500mV] := 3;
  PgaSwitch[Ch2][Pga5V    ] := 2;
  PgaSwitch[Ch2][Pga10V   ] := 1;
  PgaSwitch[Ch2][Pga20V   ] := 0;

  PgaSwitchValue[Pga1250mV] :=  1.25;
  PgaSwitchValue[Pga2500mV] :=  2.5 ;
  PgaSwitchValue[Pga5V    ] :=  5.0 ;
  PgaSwitchValue[Pga10V   ] := 10.0 ;
  PgaSwitchValue[Pga20V   ] := 20.0 ;

  ClearSRAM;

end; { tijd bepaling }



procedure StartMeasurement( MeasureSetting : MeasurementSettingsType;
                           var MeasureData : MeasurementDataType
                         );
(*
 * DESCRIPTION
 *   This routine performs 1 measurement with the TP112
 * INPUT
 *   MeasureSetting     : how to perform the measurement
 * OUTPUT
 *  MeasureData         : the measured data
 *)
var meet_getal   : integer;
    hulp_boolean : boolean;
    i            : integer;
    waarde1,
    waarde2      : real;
    wachttijd    : longint;
    InstrumentSettings : InstrumentSettingsType;
begin

  with MeasureSetting do
  begin

    SetPga( Ch1, bCoupling[Ch1], bySwitch[Ch1] );
    SetPga( Ch2, bCoupling[Ch2], bySwitch[Ch2] );
    delay(1);

    InstrumentSettings.rSampleFreq      := rSampleFreq;
    InstrumentSettings.lPostSamples     := lPostSamples;
    InstrumentSettings.lPreSamples      := lPreSamples;
    InstrumentSettings.byTrigLevel[Ch1] := byTrigLevel[Ch1];
    InstrumentSettings.byTrigLevel[Ch2] := byTrigLevel[Ch2];
    InstrumentSettings.byTrigHys        := byTrigHys;
    InstrumentSettings.bTrigSlope       := bTrigSlope;
    InstrumentSettings.byTrigSource     := nTrigSource;
  end;

  SetupForMeasurement( InstrumentSettings );

  asm
    CLI    { disable interrupts    }
  end;
  lTimerCount := 0; { clear timeout counter }
  asm
    STI    { enable interrupts     }
  end;

  repeat
   (*
    * This loop is to wait for the End Of Conversion sigal.
    * In this loop the timeout counter is monitored.
    * On interruptbase this counter is increased 18.2 times per second.
    * If the timeout value is reached, a start of measurement is forced
    * by setting the Trigger bit.
    * Waiting continues until an End Of Conversion is detected.
    *)
    if   lTimerCount >= MeasureSetting.lTimeOut
    then SetTrigbit( InstrumentSettings );
  until EOC;

  if   EOC
  then begin
         ReadDataSRAM( InstrumentSettings, MeasureData );
       end;

end; { StartMeasurement }

